// Downloaded from https://repo.progsbase.com - Code Developed Using progsbase.

function ParseORCGiro(data, ocrGiroDataReference, errorMessage){
  var records;
  var rec, str;
  var success;
  var ocr;
  var i, oppdrag;
  var linje;

  ocr = {};

  success = true;

  data = Trim(data);
  records = SplitByCharacter(data, '\n');

  /* Start record. */
  linje = CreateNumberReference(0);
  rec = records[0].string;

  if(rec[0] == 'N' && rec[1] == 'Y'){
    if(rec[2] == '0' && rec[3] == '0'){
      if(rec[4] == '0' && rec[5] == '0'){
        if(rec[6] == '1' && rec[7] == '0'){

          str = Substring(rec, 8, 8 + 8);
          ocr.avsender = nCreateNumberFromDecimalString(str);
          str = Substring(rec, 15, 15 + 7);
          ocr.forsendelsesnummer = nCreateNumberFromDecimalString(str);
          str = Substring(rec, 22, 22 + 8);
          ocr.datamottaker = nCreateNumberFromDecimalString(str);

          for(i = 31; i < 49; i = i + 1){
            if(rec[i] != '0'){
              success = false;
              errorMessage.string = "Start record FILLER must all be 0.".split('');
            }
          }
        }else{
          errorMessage.string = "Start record RECORDTYPE must be 10.".split('');
          success = false;
        }
      }else{
        errorMessage.string = "Start record FORSENDELSESTYPE must be 00.".split('');
        success = false;
      }
    }else{
      errorMessage.string = "Start record TJENESTEKODE must be 00.".split('');
      success = false;
    }
  }else{
    errorMessage.string = "Start record must rec with NY.".split('');
    success = false;
  }

  if(success){
    /* Tell oppdrag. */
    oppdrag = 0;
    linje.numberValue = 1;
    for(i = linje.numberValue; i < records.length; i = i + 1){
      rec = records[i].string;
      if(SubstringEquals(rec, 0, "NY090020".split(''))){
        oppdrag = oppdrag + 1;
      }
    }

    ocr.oppdrag = [];
    ocr.oppdrag.length = oppdrag;
    for(i = 0; i < oppdrag; i = i + 1){
      ocr.oppdrag[i] = {};
      success = ParseORCGiroOppdrag(ocr.oppdrag[i], linje, records, errorMessage);
    }
  }

  if(success){
    ocrGiroDataReference.ocrGiroData = ocr;
  }

  return success;
}
function ParseORCGiroOppdrag(oppdrag, linje, records, errorMessage){
  var i, transactions;
  var rec, str;
  var success;
  success = true;

  rec = records[linje.numberValue].string;

  /* start record oppdrag */
  if(rec[0] == 'N' && rec[1] == 'Y'){
    if(rec[2] == '0' && rec[3] == '9'){
      if(rec[4] == '0' && rec[5] == '0'){
        if(rec[6] == '2' && rec[7] == '0'){

          str = Substring(rec, 8, 8 + 9);
          oppdrag.avtaleid = nCreateNumberFromDecimalString(str);
          str = Substring(rec, 17, 17 + 7);
          oppdrag.oppdragsnummer = nCreateNumberFromDecimalString(str);
          oppdrag.oppdragskonto = Substring(rec, 24, 24 + 11);

          for(i = 35; i < 45; i = i + 1){
            if(rec[i] != '0'){
              success = false;
              errorMessage.string = "Start record FILLER must all be 0.".split('');
            }
          }
        }else{
          errorMessage.string = "Oppdrag record RECORDTYPE must be 20.".split('');
          success = false;
        }
      }else{
        errorMessage.string = "Oppdrag record OPPDRAGSTYPE must be 00.".split('');
        success = false;
      }
    }else{
      errorMessage.string = "Oppdrag record TJENESTEKODE must be 09.".split('');
      success = false;
    }
  }else{
    errorMessage.string = "Oppdrag record must start with NY.".split('');
    success = false;
  }

  /* Tell transactions */
  linje.numberValue = linje.numberValue + 1;
  transactions = 0;
  if(success){
    for(i = linje.numberValue; i < records.length; i = i + 1){
      rec = records[i].string;
      if(rec[0] == 'N' && rec[1] == 'Y'){
        if(rec[2] == '0' && rec[3] == '9'){
          if(rec[6] == '3' && rec[7] == '0'){
            transactions = transactions + 1;
          }
        }
      }
    }
  }

  oppdrag.transaksjoner = [];
  oppdrag.transaksjoner.length = transactions;

  for(i = 0; i < transactions; i = i + 1){
    oppdrag.transaksjoner[i] = {};
    success = ParseORCGiroTransaksjon(oppdrag.transaksjoner[i], linje, records, errorMessage);
  }

  return success;
}
function ParseORCGiroTransaksjon(transaksjon, linje, records, errorMessage){
  var i;
  var rec, str;
  var success;
  var transaksjonnr;
  var fortegn;
  var belop, r;

  success = true;

  rec = records[linje.numberValue].string;

  /* Belop 1 */
  if(rec[0] == 'N' && rec[1] == 'Y'){
    if(rec[2] == '0' && rec[3] == '9'){
      if(rec[6] == '3' && rec[7] == '0'){
        str = Substring(rec, 4, 4 + 2);
        transaksjon.type1 = nCreateNumberFromDecimalString(str);

        str = Substring(rec, 8, 8 + 7);
        transaksjon.nr = nCreateNumberFromDecimalString(str);

        transaksjon.dato = Substring(rec, 15, 15 + 6);

        str = Substring(rec, 21, 21 + 2);
        transaksjon.sentralid = nCreateNumberFromDecimalString(str);

        str = Substring(rec, 23, 23 + 2);
        transaksjon.dagkode = nCreateNumberFromDecimalString(str);

        str = Substring(rec, 25, 25 + 1);
        transaksjon.delavregningsnummer = nCreateNumberFromDecimalString(str);

        str = Substring(rec, 26, 26 + 5);
        transaksjon.lopenummer = nCreateNumberFromDecimalString(str);

        fortegn = rec[31];

        str = Substring(rec, 32, 32 + 17);
        belop = CreateFixedPoint15d(13, 2);
        Assign15d(belop, nCreateNumberFromDecimalString(str));
        r = CreateFixedPoint15d(13, 2);
        DivideFlooredInto15d(belop, r, Number15d(100));

        transaksjon.belop = ToString15d(belop);

        transaksjon.kid = Trim(Substring(rec, 49, 49 + 25));

        str = Substring(rec, 74, 74 + 2);
        transaksjon.kortutsteder = nCreateNumberFromDecimalString(str);

        for(i = 76; i < 80; i = i + 1){
          if(rec[i] != '0'){
            success = false;
            errorMessage.string = "Transaksjon record FILLER must all be 0.".split('');
          }
        }

        transaksjon.id = "".split('');
      }else{
        errorMessage.string = "Transaksjon record RECORDTYPE 1 must be 30.".split('');
        success = false;
      }
    }else{
      errorMessage.string = "Transaksjon record TJENESTEKODE must be 09.".split('');
      success = false;
    }
  }else{
    errorMessage.string = "Transaksjon record must start with NY.".split('');
    success = false;
  }

  /* Belop 2 */
  linje.numberValue = linje.numberValue + 1;
  rec = records[linje.numberValue].string;
  if(rec[0] == 'N' && rec[1] == 'Y'){
    if(rec[2] == '0' && rec[3] == '9'){
      if(rec[6] == '3' && rec[7] == '1'){
        str = Substring(rec, 4, 4 + 2);
        transaksjon.type2 = nCreateNumberFromDecimalString(str);

        str = Substring(rec, 8, 8 + 7);
        transaksjonnr = nCreateNumberFromDecimalString(str);

        if(transaksjonnr == transaksjon.nr){

          transaksjon.blankettnummer = Substring(rec, 15, 15 + 10);
          transaksjon.avtaleid = Substring(rec, 25, 25 + 9);

          for(i = 34; i < 41; i = i + 1){
            if(rec[i] != '0'){
              success = false;
              errorMessage.string = "Transaksjon record FILLER must all be 0.".split('');
            }
          }

          if(success){

            transaksjon.oppdragsdato = Substring(rec, 41, 41 + 6);
            transaksjon.debetkonto = Substring(rec, 47, 47 + 11);

            for(i = 58; i < 80; i = i + 1){
              if(rec[i] != '0'){
                success = false;
                errorMessage.string = "Transaksjon record FILLER must all be 0.".split('');
              }
            }
          }
        }else{
          errorMessage.string = "Belop record 2 uten samme transaksjonsnummer som belop record 1.".split('');
          success = false;
        }
      }else{
        errorMessage.string = "Transaksjon record part 2 RECORDTYPE 1 must be 31.".split('');
        success = false;
      }
    }else{
      errorMessage.string = "Transaksjon record part 2 TJENESTEKODE must be 09.".split('');
      success = false;
    }
  }else{
    errorMessage.string = "Transaksjon record part 2 must start with NY.".split('');
    success = false;
  }

  linje.numberValue = linje.numberValue + 1;

  return success;
}
function test(){
  var failures;
  failures = 0;

  failures = failures + test1();

  return failures;
}
function test1(){
  var failures;
  var errorMessage;
  var data;
  var ocrGiroDataReference;
  var success;

  failures = CreateNumberReference(0);
  errorMessage = {};

  data = "NY000010000080800322348002304070000000000000000000000000000000000000000000000000\n".split('');
  data = ConcatenateString(data, "NY090020001510150000000115061921114000000000000000000000000000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000010102218001110753000000000000052969                  0024471000000\n".split(''));
  data = ConcatenateString(data, "NY091331000000100000000008601352430000000310121810140894460000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY090088000000010000000400000000000052969010221010221010221000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY000089000000010000000600000000000052969010221000000000000000000000000000000000\n".split(''));

  ocrGiroDataReference = {};
  success = ParseORCGiro(data, ocrGiroDataReference, errorMessage);

  data = "NY000010000080800352244002304070000000000000000000000000000000000000000000000000\n".split('');
  data = ConcatenateString(data, "NY090020001510150000000115061921114000000000000000000000000000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000010402218004977774000000000000077000                  0024422000000\n".split(''));
  data = ConcatenateString(data, "NY091331000000100034188027974107500000000040221150347309350000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY090088000000010000000400000000000077000040221040221040221000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY000089000000010000000600000000000077000040221000000000000000000000000000000000\n".split(''));

  ocrGiroDataReference = {};
  success = ParseORCGiro(data, ocrGiroDataReference, errorMessage);

  data = "NY000010000080800392332002304070000000000000000000000000000000000000000000000000\n".split('');
  data = ConcatenateString(data, "NY090020001510150000000115061921114000000000000000000000000000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000010802218008107481000000000000060450                  0024547000000\n".split(''));
  data = ConcatenateString(data, "NY091331000000100035389487974105340000000080221150385741560000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000020802218008964365000000000000094900                  0024968000000\n".split(''));
  data = ConcatenateString(data, "NY091331000000200000000001802894510000000080221900405837410000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000030802218008986798000000000000023375                  0024729000000\n".split(''));
  data = ConcatenateString(data, "NY091331000000300000000008601661710000000080221858016963980000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY090088000000030000000800000000000178725080221080221080221000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY000089000000030000001000000000000178725080221000000000000000000000000000000000\n".split(''));

  ocrGiroDataReference = {};
  success = ParseORCGiro(data, ocrGiroDataReference, errorMessage);

  data = "NY000010000080800402219002304070000000000000000000000000000000000000000000000000\n".split('');
  data = ConcatenateString(data, "NY090020001510150000000115061921114000000000000000000000000000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000010902218009105188000000000000021681                  0025189000000\n".split(''));
  data = ConcatenateString(data, "NY091331000000100000000008601849280000000080221810120771140000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000020902218009105188000000000000061100                  0025197000000\n".split(''));
  data = ConcatenateString(data, "NY091331000000200000000008601849320000000080221810120771140000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY090088000000020000000600000000000082781090221090221090221000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY000089000000020000000800000000000082781090221000000000000000000000000000000000\n".split(''));

  ocrGiroDataReference = {};
  success = ParseORCGiro(data, ocrGiroDataReference, errorMessage);

  data = "NY000010000080800412254002304070000000000000000000000000000000000000000000000000\n".split('');
  data = ConcatenateString(data, "NY090020001510150000000115061921114000000000000000000000000000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000011002218010964033000000000000023099                  0025262000000\n".split(''));
  data = ConcatenateString(data, "NY091331000000100000000009750548970000000100221975007277330000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY090088000000010000000400000000000023099100221100221100221000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY000089000000010000000600000000000023099100221000000000000000000000000000000000\n".split(''));

  ocrGiroDataReference = {};
  success = ParseORCGiro(data, ocrGiroDataReference, errorMessage);

  data = "NY000010000080800170031000102000000000000000000000000000000000000000000000000000\n".split('');
  data = ConcatenateString(data, "NY090020001767676000000199991111111000000000000000000000000000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09103000000012403040124112345000000000000044000           33000083672049000000\n".split(''));
  data = ConcatenateString(data, "NY091031000000160004322610945611540000000230304888810111280000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09103000000022403040124112345000000000000008000           33000158843046000000\n".split(''));
  data = ConcatenateString(data, "NY091031000000260004333650944552120000000230304888810111360000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09103000000032403040124112345000000000000150000           33000163680003000000\n".split(''));
  data = ConcatenateString(data, "NY091031000000360004332510965888510000000230304888810111440000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09103000000042403040124112345000000000000150000           33000164573025000000\n".split(''));
  data = ConcatenateString(data, "NY091031000000460004336140945666540000000230304888810111520000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09103000000052403040124112345000000000000350000           33000167698019000000\n".split(''));
  data = ConcatenateString(data, "NY091031000000560004336710945141610000000230304888810111520000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09103000000062403040124112345000000000000160000           33000170130034000000\n".split(''));
  data = ConcatenateString(data, "NY091031000000660004322160981454110000000230304888810111600000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09103000000072403040124112345000000000000120000           33000173783060000000\n".split(''));
  data = ConcatenateString(data, "NY091031000000760004321650955451220000000230304888810111790000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000082403040124967890000000000000090000           01000000258037000000\n".split(''));
  data = ConcatenateString(data, "NY091331000000800000000006001011400000000240304888810111870000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000092403040124967890000000000000010000           01000000368026000000\n".split(''));
  data = ConcatenateString(data, "NY091331000000900000000006105011380000000240304888810111870000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000102403040124967890000000000000175000           01000000407048000000\n".split(''));
  data = ConcatenateString(data, "NY091331000001000000000006001011400000000240304888810111870000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000112403040124967890000000000000150000           01000000488022000000\n".split(''));
  data = ConcatenateString(data, "NY091331000001100000000006105011380000000240304888810111870000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000122403040124967890000000000000015000           01000000608025000000\n".split(''));
  data = ConcatenateString(data, "NY091331000001200000000006105011380000000240304888810111870000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000132403040124967890000000000000036000           01000000717040000000\n".split(''));
  data = ConcatenateString(data, "NY091331000001300000000006001011400000000240304888810111870000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000142403040124967890000000000000060000           01000000768035000000\n".split(''));
  data = ConcatenateString(data, "NY091331000001400000000006105011380000000240304888810111870000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000152403040124967890-00000000000025000           44000366370078000000\n".split(''));
  data = ConcatenateString(data, "NY091331000001500000000006001011400000000240304888810111870000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000162403040124967890-00000000000040000           44000366980090000000\n".split(''));
  data = ConcatenateString(data, "NY091331000001600000000006001011400000000240304888810111870000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000172403040124967890-00000000000150000           44000007860156000000\n".split(''));
  data = ConcatenateString(data, "NY091331000001700000000006105011380000000240304888810111870000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000182403040124967890000000000000030000           44000000380020000000\n".split(''));
  data = ConcatenateString(data, "NY091331000001800000000005802017930000000240304888810111950000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000192403040124967890000000000000010000           44000000560084000000\n".split(''));
  data = ConcatenateString(data, "NY091331000001900000000005802017930000000240304888810111950000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000202403040124967890000000000000015000           44000000640092000000\n".split(''));
  data = ConcatenateString(data, "NY091331000002000000000005802017930000000240304888810111950000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000212403040124967890000000000000045000           44000000730042000000\n".split(''));
  data = ConcatenateString(data, "NY091331000002100000000005802017930000000240304888810111950000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000222403040124967890000000000000075000           44000001060027000000\n".split(''));
  data = ConcatenateString(data, "NY091331000002200000000005986667840000000240304888810111010000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000232403040124967890000000000000085000           44000001100070000000\n".split(''));
  data = ConcatenateString(data, "NY091331000002300000000005986667840000000240304888810111010000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY090088000000230000004800000000001563000240304240304240304000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY000089000000230000005000000000001563000240304000000000000000000000000000000000\n".split(''));

  ocrGiroDataReference = {};
  success = ParseORCGiro(data, ocrGiroDataReference, errorMessage);

  data = "NY000010000080800422236002304070000000000000000000000000000000000000000000000000\n".split('');
  data = ConcatenateString(data, "NY090020001510150000000115061921114000000000000000000000000000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000011102218011105839000000000000097475                  0024786000000\n".split(''));
  data = ConcatenateString(data, "NY091331000000100000000008601475840000000100221810140894460000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000021102218011105839000000000000046929                  0025296000000\n".split(''));
  data = ConcatenateString(data, "NY091331000000200000000008601475810000000100221810140894460000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY090088000000020000000600000000000144404110221110221110221000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY000089000000020000000800000000000144404110221000000000000000000000000000000000\n".split(''));

  ocrGiroDataReference = {};
  success = ParseORCGiro(data, ocrGiroDataReference, errorMessage);

  data = "NY000010000080800432279002304070000000000000000000000000000000000000000000000000\n".split('');
  data = ConcatenateString(data, "NY090020001510150000000115061921114000000000000000000000000000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000011202218012106273000000000000026000                  0025338000000\n".split(''));
  data = ConcatenateString(data, "NY091331000000100000000008601166960000000110221810120771140000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000021202218012909104000000000000017380                  0025361000000\n".split(''));
  data = ConcatenateString(data, "NY091331000000200002113807974557090000000120221150345968210000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY090088000000020000000600000000000043380120221120221120221000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY000089000000020000000800000000000043380120221000000000000000000000000000000000\n".split(''));

  ocrGiroDataReference = {};
  success = ParseORCGiro(data, ocrGiroDataReference, errorMessage);

  data = "NY000010000080801472183002304070000000000000000000000000000000000000000000000000\n".split('');
  data = ConcatenateString(data, "NY090020001510150000000115061921114000000000000000000000000000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000012705218027933772000000000000030375                  0027169000000\n".split(''));
  data = ConcatenateString(data, "NY091331000000100013875860883055260000000270521626306534820000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000022705218027947997000000000000047250                  0026328000000\n".split(''));
  data = ConcatenateString(data, "NY091331000000200013206427974551870000000270521150632604680000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000032705218027947997000000000000047250                  0027193000000\n".split(''));
  data = ConcatenateString(data, "NY091331000000300013206427974551870000000270521150632604680000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY090088000000030000000800000000000124875270521270521270521000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY000089000000030000001000000000000124875270521000000000000000000000000000000000\n".split(''));

  ocrGiroDataReference = {};
  success = ParseORCGiro(data, ocrGiroDataReference, errorMessage);

  data = "NY000010000080801512316002304070000000000000000000000000000000000000000000000000\n".split('');
  data = ConcatenateString(data, "NY090020001510150000000115061921114000000000000000000000000000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000013105218031108806000000000000028191                  0027854000000\n".split(''));
  data = ConcatenateString(data, "NY091331000000100000000006676540530000000310521608206324380000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY090088000000010000000400000000000028191310521310521310521000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY000089000000010000000600000000000028191310521000000000000000000000000000000000\n".split(''));

  ocrGiroDataReference = {};
  success = ParseORCGiro(data, ocrGiroDataReference, errorMessage);

  data = "NY000010000080803482094002304070000000000000000000000000000000000000000000000000\n".split('');
  data = ConcatenateString(data, "NY090020001510150000000115061921114000000000000000000000000000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000011412218014106879000000000000750000                  0034157000000\n".split(''));
  data = ConcatenateString(data, "NY091331000000100000000008601414700000000131221810121953030000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY090088000000010000000400000000000750000141221141221141221000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY000089000000010000000600000000000750000141221000000000000000000000000000000000\n".split(''));

  ocrGiroDataReference = {};
  success = ParseORCGiro(data, ocrGiroDataReference, errorMessage);

  data = "NY000010000080803632073002304070000000000000000000000000000000000000000000000000\n".split('');
  data = ConcatenateString(data, "NY090020001510150000000115061921114000000000000000000000000000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000012912218029952241000000000000505778                  0034215000000\n".split(''));
  data = ConcatenateString(data, "NY091331000000100000000003201994510000000291221320140633570000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000022912218029952241000000000000708875                  0034629000000\n".split(''));
  data = ConcatenateString(data, "NY091331000000200002259357974401300000000291221150394173050000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY090088000000020000000600000000001214653291221291221291221000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY000089000000020000000800000000001214653291221000000000000000000000000000000000\n".split(''));

  data = "NY000010000080800032160002304070000000000000000000000000000000000000000000000000\n".split('');
  data = ConcatenateString(data, "NY090020001510150000000115061921114000000000000000000000000000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000010301228003109674000000000001361750                  0034272000000\n".split(''));
  data = ConcatenateString(data, "NY091331000000100005620117974104500000000030122150624422010000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000020301228003109674000000000000167250                  0034314000000\n".split(''));
  data = ConcatenateString(data, "NY091331000000200000000003910394510000000030122391063971670000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000030301228003109674000000000000024875                  0034322000000\n".split(''));
  data = ConcatenateString(data, "NY091331000000300005709857974200870000000030122150338022230000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000040301228003109674000000000000047250                  0034405000000\n".split(''));
  data = ConcatenateString(data, "NY091331000000400000000006676558290000000030122607505265380000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000050301228003109674000000000000047250                  0034413000000\n".split(''));
  data = ConcatenateString(data, "NY091331000000500000000001802394510000000010122182285315250000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000060301228003109674000000000000060450                  0034447000000\n".split(''));
  data = ConcatenateString(data, "NY091331000000600014687447974200620000000030122150326530700000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000070301228003109674000000000000197000                  0034454000000\n".split(''));
  data = ConcatenateString(data, "NY091331000000700003503507974208740000000030122150314275810000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000080301228003109674000000000000778250                  0034504000000\n".split(''));
  data = ConcatenateString(data, "NY091331000000800000000009055394510000000010122905305095820000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000090301228003109674000000000000437125                  0034512000000\n".split(''));
  data = ConcatenateString(data, "NY091331000000900000000008601043930000000020122860135885210000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000100301228003109674000000000000047250                  0034553000000\n".split(''));
  data = ConcatenateString(data, "NY091331000001000015508230889843860000000030122150618972800000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000110301228003109674000000000000078500                  0034561000000\n".split(''));
  data = ConcatenateString(data, "NY091331000001100007372257974206780000000030122150359045970000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000120301228003109674000000000000323375                  0034579000000\n".split(''));
  data = ConcatenateString(data, "NY091331000001200000000006676535130000000030122608206324380000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000130301228003109674000000000000023375                  0034595000000\n".split(''));
  data = ConcatenateString(data, "NY091331000001300000000008601201230000000020122858016963980000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000140301228003109674000000000000035983                  0034611000000\n".split(''));
  data = ConcatenateString(data, "NY091331000001400005714697974106660000000030122150618012750000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000150301228003109674000000000000862525                  0034652000000\n".split(''));
  data = ConcatenateString(data, "NY091331000001500000000008601114400000000020122810140894460000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000160301228003109674000000000000061100                  0034736000000\n".split(''));
  data = ConcatenateString(data, "NY091331000001600000000008601040070000000020122810120771140000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000170301228003109674000000000000033275                  0034801000000\n".split(''));
  data = ConcatenateString(data, "NY091331000001700013917877974201960000000030122150330979880000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000180301228003109674000000000000075025                  0034819000000\n".split(''));
  data = ConcatenateString(data, "NY091331000001800000000001802394510000000010122900405837410000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000190301228003109674000000000000060450                  0034835000000\n".split(''));
  data = ConcatenateString(data, "NY091331000001900000000006676513230000000030122601005167750000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000200301228003109674000000000000027075                  0034850000000\n".split(''));
  data = ConcatenateString(data, "NY091331000002000000000006676010430000000030122600505990700000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000210301228003109674000000000000322000                  0034868000000\n".split(''));
  data = ConcatenateString(data, "NY091331000002100004720427974102130000000030122150398679980000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000220301228003109674000000000000085760                  0034876000000\n".split(''));
  data = ConcatenateString(data, "NY091331000002200000000002544394510000000010122247025549570000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000230301228003109674000000000000047250                  0034892000000\n".split(''));
  data = ConcatenateString(data, "NY091331000002300000000004201394510000000010122421320079710000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000240301228003914839000000000000023375                  0034355000000\n".split(''));
  data = ConcatenateString(data, "NY091331000002400013875860880274290000000030122626306534820000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000250301228003914839000000000000023375                  0034744000000\n".split(''));
  data = ConcatenateString(data, "NY091331000002500000000002050382280000000030122205039182000000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000260301228003914839000000000000047250                  0034785000000\n".split(''));
  data = ConcatenateString(data, "NY091331000002600007237077974102220000000030122150338933930000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000270301228003914839000000000000033275                  0034793000000\n".split(''));
  data = ConcatenateString(data, "NY091331000002700007141047974409440000000030122160258377740000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000280301228003914839000000000000047250                  0034959000000\n".split(''));
  data = ConcatenateString(data, "NY091331000002800007348827974404430000000030122150632048270000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000290301228003935623000000000000094875                  0034231000000\n".split(''));
  data = ConcatenateString(data, "NY091331000002900000000002811394510000000030122280132853240000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000300301228003935623000000000000041750                  0034694000000\n".split(''));
  data = ConcatenateString(data, "NY091331000003000036271857974102010000000030122502805149150000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000310301228003997144000000000002515650                  0034223000000\n".split(''));
  data = ConcatenateString(data, "NY091331000003100000000008601236110000000030122810121953030000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000320301228003997144000000000000124300                  0034280000000\n".split(''));
  data = ConcatenateString(data, "NY091331000003200000000006676572570000000030122607905509890000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000330301228003997144000000000000070625                  0034660000000\n".split(''));
  data = ConcatenateString(data, "NY091331000003300000000006676041350000000030122600505946800000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000340301228003997144000000000000072125                  0034728000000\n".split(''));
  data = ConcatenateString(data, "NY091331000003400000000006676041350000000030122600505946800000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY09133000000350301228003997144000000000000047250                  0034926000000\n".split(''));
  data = ConcatenateString(data, "NY091331000003500000000008601262050000000030122815017705900000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY090088000000350000007200000000008345243030122030122030122000000000000000000000\n".split(''));
  data = ConcatenateString(data, "NY000089000000350000007400000000008345243030122000000000000000000000000000000000\n".split(''));

  ocrGiroDataReference = {};
  success = ParseORCGiro(data, ocrGiroDataReference, errorMessage);

  return failures.numberValue;
}
function nCreateStringScientificNotationDecimalFromNumber(decimal){
  var mantissaReference, exponentReference;
  var multiplier, inc;
  var exponent;
  var done, isPositive;
  var result;

  mantissaReference = {};
  exponentReference = {};
  result = [];
  result.length = 0;
  done = false;
  exponent = 0;

  if(decimal < 0){
    isPositive = false;
    decimal =  -decimal;
  }else{
    isPositive = true;
  }

  if(decimal == 0){
    done = true;
  }

  if( !done ){
    multiplier = 0;
    inc = 0;

    if(decimal < 1){
      multiplier = 10;
      inc =  -1;
    }else if(decimal >= 10){
      multiplier = 0.1;
      inc = 1;
    }else{
      done = true;
    }

    if( !done ){
      exponent = Math.round(Math.log10(decimal));
      exponent = Math.min(99, exponent);
      exponent = Math.max( -99, exponent);

      decimal = decimal/Math.pow(10, exponent);

      /* Adjust */
      for(; (decimal >= 10 || decimal < 1) && Math.abs(exponent) < 99; ){
        decimal = decimal*multiplier;
        exponent = exponent + inc;
      }
    }
  }

  nCreateStringFromNumberWithCheck(decimal, 10, mantissaReference);

  nCreateStringFromNumberWithCheck(exponent, 10, exponentReference);

  if( !isPositive ){
    result = AppendString(result, "-".split(''));
  }

  result = AppendString(result, mantissaReference.string);
  result = AppendString(result, "e".split(''));
  result = AppendString(result, exponentReference.string);

  return result;
}
function nCreateStringDecimalFromNumber(decimal){
  var stringReference;

  stringReference = {};

  /* This will succeed because base = 10. */
  nCreateStringFromNumberWithCheck(decimal, 10, stringReference);

  return stringReference.string;
}
function nCreateStringFromNumberWithCheck(decimal, base, stringReference){
  var string;
  var maximumDigits;
  var digitPosition;
  var hasPrintedPoint, isPositive;
  var i, d;
  var success;
  var characterReference;
  var c;

  string = CreateDynamicArrayCharacters();
  isPositive = true;

  if(decimal < 0){
    isPositive = false;
    decimal =  -decimal;
  }

  if(decimal == 0){
    DynamicArrayAddCharacter(string, '0');
    success = true;
  }else{
    characterReference = {};

    if(IsInteger(base)){
      success = true;

      maximumDigits = nGetMaximumDigitsForBase(base);

      digitPosition = nGetFirstDigitPosition(decimal, base);

      decimal = Math.round(decimal*Math.pow(base, maximumDigits - digitPosition - 1));

      hasPrintedPoint = false;

      if( !isPositive ){
        DynamicArrayAddCharacter(string, '-');
      }

      /* Print leading zeros. */
      if(digitPosition < 0){
        DynamicArrayAddCharacter(string, '0');
        DynamicArrayAddCharacter(string, '.');
        hasPrintedPoint = true;
        for(i = 0; i <  -digitPosition - 1; i = i + 1){
          DynamicArrayAddCharacter(string, '0');
        }
      }

      /* Print number. */
      for(i = 0; i < maximumDigits && success; i = i + 1){
        d = Math.floor(decimal/Math.pow(base, maximumDigits - i - 1));

        if(d >= base){
          d = base - 1;
        }

        if( !hasPrintedPoint  && digitPosition - i + 1 == 0){
          if(decimal != 0){
            DynamicArrayAddCharacter(string, '.');
          }
          hasPrintedPoint = true;
        }

        if(decimal == 0 && hasPrintedPoint){
        }else{
          success = nGetSingleDigitCharacterFromNumberWithCheck(d, base, characterReference);
          if(success){
            c = characterReference.characterValue;
            DynamicArrayAddCharacter(string, c);
          }
        }

        if(success){
          decimal = decimal - d*Math.pow(base, maximumDigits - i - 1);
        }
      }

      if(success){
        /* Print trailing zeros. */
        for(i = 0; i < digitPosition - maximumDigits + 1; i = i + 1){
          DynamicArrayAddCharacter(string, '0');
        }
      }
    }else{
      success = false;
    }
  }

  if(success){
    stringReference.string = DynamicArrayCharactersToArray(string);
    FreeDynamicArrayCharacters(string);
  }

  /* Done */
  return success;
}
function nGetMaximumDigitsForBase(base){
  var t;

  t = Math.pow(10, 15);
  return Math.floor(Math.log10(t)/Math.log10(base));
}
function nGetFirstDigitPosition(decimal, base){
  var power;
  var t;

  power = Math.ceil(Math.log10(decimal)/Math.log10(base));

  t = decimal*Math.pow(base,  -power);
  if(t < base && t >= 1){
  }else if(t >= base){
    power = power + 1;
  }else if(t < 1){
    power = power - 1;
  }

  return power;
}
function nGetSingleDigitCharacterFromNumberWithCheck(c, base, characterReference){
  var numberTable;
  var success;

  numberTable = nGetDigitCharacterTable();

  if(c < base || c < numberTable.length){
    success = true;
    characterReference.characterValue = numberTable[c];
  }else{
    success = false;
  }

  return success;
}
function nGetDigitCharacterTable(){
  var numberTable;

  numberTable = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ".split('');

  return numberTable;
}
function nCreateNumberFromDecimalStringWithCheck(string, decimalReference, errorMessage){
  return nCreateNumberFromStringWithCheck(string, 10, decimalReference, errorMessage);
}
function nCreateNumberFromDecimalString(string){
  var doubleReference;
  var stringReference;
  var number;

  doubleReference = CreateNumberReference(0);
  stringReference = CreateStringReference("".split(''));
  nCreateNumberFromStringWithCheck(string, 10, doubleReference, stringReference);
  number = doubleReference.numberValue;

  delete(doubleReference);
  delete(stringReference);

  return number;
}
function nCreateNumberFromStringWithCheck(string, base, numberReference, errorMessage){
  var success;
  var numberIsPositive, exponentIsPositive;
  var beforePoint, afterPoint, exponent;

  numberIsPositive = CreateBooleanReference(true);
  exponentIsPositive = CreateBooleanReference(true);
  beforePoint = {};
  afterPoint = {};
  exponent = {};

  if(base >= 2 && base <= 36){
    success = nExtractPartsFromNumberString(string, base, numberIsPositive, beforePoint, afterPoint, exponentIsPositive, exponent, errorMessage);

    if(success){
      numberReference.numberValue = nCreateNumberFromParts(base, numberIsPositive.booleanValue, beforePoint.numberArray, afterPoint.numberArray, exponentIsPositive.booleanValue, exponent.numberArray);
    }
  }else{
    success = false;
    errorMessage.string = "Base must be from 2 to 36.".split('');
  }

  return success;
}
function nCreateNumberFromParts(base, numberIsPositive, beforePoint, afterPoint, exponentIsPositive, exponent){
  var n, i, p, e;

  n = 0;

  for(i = 0; i < beforePoint.length; i = i + 1){
    p = beforePoint[beforePoint.length - i - 1];

    n = n + p*Math.pow(base, i);
  }

  for(i = 0; i < afterPoint.length; i = i + 1){
    p = afterPoint[i];

    n = n + p*Math.pow(base,  -(i + 1));
  }

  if(exponent.length > 0){
    e = 0;
    for(i = 0; i < exponent.length; i = i + 1){
      p = exponent[exponent.length - i - 1];

      e = e + p*Math.pow(base, i);
    }

    if( !exponentIsPositive ){
      e =  -e;
    }

    n = n*Math.pow(base, e);
  }

  if( !numberIsPositive ){
    n =  -n;
  }

  return n;
}
function nExtractPartsFromNumberString(n, base, numberIsPositive, beforePoint, afterPoint, exponentIsPositive, exponent, errorMessages){
  var i, j, count;
  var success, done, complete;

  i = 0;
  complete = false;

  if(i < n.length){
    if(n[i] == '-'){
      numberIsPositive.booleanValue = false;
      i = i + 1;
    }else if(n[i] == '+'){
      numberIsPositive.booleanValue = true;
      i = i + 1;
    }

    success = true;
  }else{
    success = false;
    errorMessages.string = "Number cannot have length zero.".split('');
  }

  if(success){
    done = false;
    count = 0;
    for(; i + count < n.length &&  !done ; ){
      if(nCharacterIsNumberCharacterInBase(n[i + count], base)){
        count = count + 1;
      }else{
        done = true;
      }
    }

    if(count >= 1){
      beforePoint.numberArray = [];
      beforePoint.numberArray.length = count;

      for(j = 0; j < count; j = j + 1){
        beforePoint.numberArray[j] = nGetNumberFromNumberCharacterForBase(n[i + j], base);
      }

      i = i + count;

      if(i < n.length){
        success = true;
      }else{
        afterPoint.numberArray = [];
        afterPoint.numberArray.length = 0;
        exponent.numberArray = [];
        exponent.numberArray.length = 0;
        success = true;
        complete = true;
      }
    }else{
      success = false;
      errorMessages.string = "Number must have at least one number after the optional sign.".split('');
    }
  }

  if(success &&  !complete ){
    if(n[i] == '.'){
      i = i + 1;

      if(i < n.length){
        done = false;
        count = 0;
        for(; i + count < n.length &&  !done ; ){
          if(nCharacterIsNumberCharacterInBase(n[i + count], base)){
            count = count + 1;
          }else{
            done = true;
          }
        }

        if(count >= 1){
          afterPoint.numberArray = [];
          afterPoint.numberArray.length = count;

          for(j = 0; j < count; j = j + 1){
            afterPoint.numberArray[j] = nGetNumberFromNumberCharacterForBase(n[i + j], base);
          }

          i = i + count;

          if(i < n.length){
            success = true;
          }else{
            exponent.numberArray = [];
            exponent.numberArray.length = 0;
            success = true;
            complete = true;
          }
        }else{
          success = false;
          errorMessages.string = "There must be at least one digit after the decimal point.".split('');
        }
      }else{
        success = false;
        errorMessages.string = "There must be at least one digit after the decimal point.".split('');
      }
    }else if(base <= 14 && (n[i] == 'e' || n[i] == 'E')){
      if(i < n.length){
        success = true;
        afterPoint.numberArray = [];
        afterPoint.numberArray.length = 0;
      }else{
        success = false;
        errorMessages.string = "There must be at least one digit after the exponent.".split('');
      }
    }else{
      success = false;
      errorMessages.string = "Expected decimal point or exponent symbol.".split('');
    }
  }

  if(success &&  !complete ){
    if(base <= 14 && (n[i] == 'e' || n[i] == 'E')){
      i = i + 1;

      if(i < n.length){
        if(n[i] == '-'){
          exponentIsPositive.booleanValue = false;
          i = i + 1;
        }else if(n[i] == '+'){
          exponentIsPositive.booleanValue = true;
          i = i + 1;
        }

        if(i < n.length){
          done = false;
          count = 0;
          for(; i + count < n.length &&  !done ; ){
            if(nCharacterIsNumberCharacterInBase(n[i + count], base)){
              count = count + 1;
            }else{
              done = true;
            }
          }

          if(count >= 1){
            exponent.numberArray = [];
            exponent.numberArray.length = count;

            for(j = 0; j < count; j = j + 1){
              exponent.numberArray[j] = nGetNumberFromNumberCharacterForBase(n[i + j], base);
            }

            i = i + count;

            if(i == n.length){
              success = true;
            }else{
              success = false;
              errorMessages.string = "There cannot be any characters past the exponent of the number.".split('');
            }
          }else{
            success = false;
            errorMessages.string = "There must be at least one digit after the decimal point.".split('');
          }
        }else{
          success = false;
          errorMessages.string = "There must be at least one digit after the exponent symbol.".split('');
        }
      }else{
        success = false;
        errorMessages.string = "There must be at least one digit after the exponent symbol.".split('');
      }
    }else{
      success = false;
      errorMessages.string = "Expected exponent symbol.".split('');
    }
  }

  return success;
}
function nGetNumberFromNumberCharacterForBase(c, base){
  var numberTable;
  var i;
  var position;

  numberTable = nGetDigitCharacterTable();
  position = 0;

  for(i = 0; i < base; i = i + 1){
    if(numberTable[i] == c){
      position = i;
    }
  }

  return position;
}
function nCharacterIsNumberCharacterInBase(c, base){
  var numberTable;
  var i;
  var found;

  numberTable = nGetDigitCharacterTable();
  found = false;

  for(i = 0; i < base; i = i + 1){
    if(numberTable[i] == c){
      found = true;
    }
  }

  return found;
}
function nStringToNumberArray(str){
  var numberArrayReference;
  var stringReference;
  var numbers;

  numberArrayReference = {};
  stringReference = {};

  nStringToNumberArrayWithCheck(str, numberArrayReference, stringReference);

  numbers = numberArrayReference.numberArray;

  delete(numberArrayReference);
  delete(stringReference);

  return numbers;
}
function nStringToNumberArrayWithCheck(str, numberArrayReference, errorMessage){
  var numberStrings;
  var numbers;
  var i;
  var numberString, trimmedNumberString;
  var success;
  var numberReference;

  numberStrings = SplitByString(str, ",".split(''));

  numbers = [];
  numbers.length = numberStrings.length;
  success = true;
  numberReference = {};

  for(i = 0; i < numberStrings.length; i = i + 1){
    numberString = numberStrings[i].string;
    trimmedNumberString = Trim(numberString);
    success = nCreateNumberFromDecimalStringWithCheck(trimmedNumberString, numberReference, errorMessage);
    numbers[i] = numberReference.numberValue;

    FreeStringReference(numberStrings[i]);
    delete(trimmedNumberString);
  }

  delete(numberStrings);
  delete(numberReference);

  numberArrayReference.numberArray = numbers;

  return success;
}
function CreateBooleanReference(value){
  var ref;

  ref = {};
  ref.booleanValue = value;

  return ref;
}
function CreateBooleanArrayReference(value){
  var ref;

  ref = {};
  ref.booleanArray = value;

  return ref;
}
function CreateBooleanArrayReferenceLengthValue(lengthx, value){
  var ref;
  var i;

  ref = {};
  ref.booleanArray = [];
  ref.booleanArray.length = lengthx;

  for(i = 0; i < lengthx; i = i + 1){
    ref.booleanArray[i] = value;
  }

  return ref;
}
function FreeBooleanArrayReference(booleanArrayReference){
  delete(booleanArrayReference.booleanArray);
  delete(booleanArrayReference);
}
function CreateCharacterReference(value){
  var ref;

  ref = {};
  ref.characterValue = value;

  return ref;
}
function CreateNumberReference(value){
  var ref;

  ref = {};
  ref.numberValue = value;

  return ref;
}
function CreateNumberArrayReference(value){
  var ref;

  ref = {};
  ref.numberArray = value;

  return ref;
}
function CreateNumberArrayReferenceLengthValue(lengthx, value){
  var ref;
  var i;

  ref = {};
  ref.numberArray = [];
  ref.numberArray.length = lengthx;

  for(i = 0; i < lengthx; i = i + 1){
    ref.numberArray[i] = value;
  }

  return ref;
}
function FreeNumberArrayReference(numberArrayReference){
  delete(numberArrayReference.numberArray);
  delete(numberArrayReference);
}
function CreateStringReference(value){
  var ref;

  ref = {};
  ref.string = value;

  return ref;
}
function CreateStringReferenceLengthValue(lengthx, value){
  var ref;
  var i;

  ref = {};
  ref.string = [];
  ref.string.length = lengthx;

  for(i = 0; i < lengthx; i = i + 1){
    ref.string[i] = value;
  }

  return ref;
}
function FreeStringReference(stringReference){
  delete(stringReference.string);
  delete(stringReference);
}
function CreateStringArrayReference(strings){
  var ref;

  ref = {};
  ref.stringArray = strings;

  return ref;
}
function CreateStringArrayReferenceLengthValue(lengthx, value){
  var ref;
  var i;

  ref = {};
  ref.stringArray = [];
  ref.stringArray.length = lengthx;

  for(i = 0; i < lengthx; i = i + 1){
    ref.stringArray[i] = CreateStringReference(value);
  }

  return ref;
}
function FreeStringArrayReference(stringArrayReference){
  var i;

  for(i = 0; i < stringArrayReference.stringArray.length; i = i + 1){
    delete(stringArrayReference.stringArray[i]);
  }
  delete(stringArrayReference.stringArray);
  delete(stringArrayReference);
}
function CreateFixedPoint30d(digitsBeforeDecimalPoint, digitsAfterDecimalPoint){
  var fp;

  fp = {};
  fp.digitsBeforeDecimalPoint = digitsBeforeDecimalPoint;
  fp.digitsAfterDecimalPoint = digitsAfterDecimalPoint;
  fp.part1 = 0;
  fp.part2 = 0;

  return fp;
}
function CreateFixedPoint15d(digitsBeforeDecimalPoint, digitsAfterDecimalPoint){
  var fp;

  fp = {};
  fp.digitsBeforeDecimalPoint = digitsBeforeDecimalPoint;
  fp.digitsAfterDecimalPoint = digitsAfterDecimalPoint;
  fp.number = 0;

  return fp;
}
function ToNumber15d(n){
  return n.number;
}
function Number15d(number){
  var fp;

  fp = {};
  fp.digitsBeforeDecimalPoint = 7;
  fp.digitsAfterDecimalPoint = 7;
  fp.number = number;

  return fp;
}
function Assign15d(fp, number){
  var success;

  success =  !WillOverflow15d(fp, number) ;
  success = success && FixedPointIsValid15d(fp);

  if(success){
    fp.number = number;
    fp.number = RoundToDigits(fp.number, fp.digitsAfterDecimalPoint);
  }

  return success;
}
function Assign15dFloor(fp, number){
  var success;

  success =  !WillOverflow15d(fp, number) ;
  success = success && FixedPointIsValid15d(fp);

  if(success){
    fp.number = number;
    fp.number = FloorToDigits(fp.number, fp.digitsAfterDecimalPoint);
  }

  return success;
}
function FixedPointIsValid15d(fp){
  var valid;

  if(IsInteger(fp.digitsAfterDecimalPoint) && IsInteger(fp.digitsBeforeDecimalPoint)){
    if(fp.digitsBeforeDecimalPoint >= 0 && fp.digitsBeforeDecimalPoint <= 15){
      if(fp.digitsAfterDecimalPoint >= 0 && fp.digitsAfterDecimalPoint <= 15){
        if(fp.digitsBeforeDecimalPoint + fp.digitsAfterDecimalPoint <= 15){
          if(fp.digitsBeforeDecimalPoint + fp.digitsAfterDecimalPoint > 0){
            valid = true;
          }else{
            valid = false;
          }
        }else{
          valid = false;
        }
      }else{
        valid = false;
      }
    }else{
      valid = false;
    }
  }else{
    valid = false;
  }

  return valid;
}
function WillOverflow15d(fp, number){
  var overflow;

  if(Math.abs(number) < Math.pow(10, fp.digitsBeforeDecimalPoint)){
    overflow = false;
  }else{
    overflow = true;
  }

  return overflow;
}
function RoundToDigits(element, digitsAfterPoint){
  return Round(element*Math.pow(10, digitsAfterPoint))/Math.pow(10, digitsAfterPoint);
}
function FloorToDigits(value, digits){
  return Math.floor(value*Math.pow(10, digits))/Math.pow(10, digits);
}
function ToString15d(fp){
  var string;
  var digits;
  var digitPosition;
  var i, d, decimal;
  var characterReference;

  string = [];
  string.length = 1 + fp.digitsBeforeDecimalPoint + 1 + fp.digitsAfterDecimalPoint;

  decimal = fp.number*Math.pow(10, fp.digitsAfterDecimalPoint);

  if(decimal < 0){
    decimal =  -decimal;
    string[0] = '-';
  }else{
    string[0] = '+';
  }

  decimal = Round(decimal);

  characterReference = {};

  digits = fp.digitsBeforeDecimalPoint + fp.digitsAfterDecimalPoint;
  digitPosition = 1;

  for(i = 0; i < digits; i = i + 1){
    if(i == fp.digitsBeforeDecimalPoint){
      string[digitPosition] = '.';

      digitPosition = digitPosition + 1;
    }

    d = Math.floor(decimal/Math.pow(10, digits - i - 1));
    d = d%10;

    nGetSingleDigitCharacterFromNumberWithCheck(d, 10, characterReference);
    string[digitPosition] = characterReference.characterValue;

    digitPosition = digitPosition + 1;
  }

  delete(characterReference);

  return string;
}
function Add15d(a, b, c){
  return Assign15d(a, b.number + c.number);
}
function AddInto15d(a, b){
  return Add15d(a, a, b);
}
function Subtract15d(a, b, c){
  return Assign15d(a, b.number - c.number);
}
function SubtractInto15d(a, b){
  return Subtract15d(a, a, b);
}
function Multiply15d(a, b, c){
  return Assign15d(a, b.number*c.number);
}
function MultiplyInto15d(a, b){
  return Multiply15d(a, a, b);
}
function DivideFloored15d(q, r, a, b){
  var success;
  var x, xDivisor, xDividend;

  if(b.number != 0){
    xDivisor = Round(a.number*Math.pow(10, q.digitsAfterDecimalPoint)*Math.pow(10, q.digitsAfterDecimalPoint));
    xDividend = Round(b.number*Math.pow(10, q.digitsAfterDecimalPoint));
    x = Math.floor(xDivisor/xDividend);
    x = x/Math.pow(10, q.digitsAfterDecimalPoint);
    success = Assign15d(q, x);
    Multiply15d(r, q, b);
    Subtract15d(r, a, r);
  }else{
    success = false;
  }

  return success;
}
function DivideFlooredInto15d(a, r, b){
  return DivideFloored15d(a, r, a, b);
}
function Negate15d(a){
  a.number =  -a.number;
}
function Positive15d(a){
  a.number =  +a.number;
}
function Factorial15d(x){
  var success;

  if(x.number >= 0){
    success = Assign15d(x, Factorial(x.number));
  }else{
    success = false;
  }

  return success;
}
function Round15d(x){
  return Assign15d(x, Round(x.number));
}
function BankersRound15d(x){
  return Assign15d(x, BankersRound(x.number));
}
function Ceil15d(x){
  return Assign15d(x, Ceil(x.number));
}
function Floor15d(x){
  return Assign15d(x, Math.floor(x.number));
}
function Truncate15d(x){
  x.number = Truncate(x.number);
}
function Absolute15d(x){
  x.number = Math.abs(x.number);
}
function Logarithm15d(x){
  var success;

  if(x.number > 0){
    success = Assign15d(x, Logarithm(x.number));
  }else{
    success = false;
  }

  return success;
}
function NaturalLogarithm15d(x){
  var success;

  if(x.number > 0){
    success = Assign15d(x, NaturalLogarithm(x.number));
  }else{
    success = false;
  }

  return success;
}
function Sin15d(x){
  return Assign15d(x, Sin(x.number));
}
function Cos15d(x){
  return Assign15d(x, Cos(x.number));
}
function Tan15d(x){
  return Assign15d(x, Tan(x.number));
}
function Asin15d(x){
  var success;

  if(x.number >=  -1 && x.number <= 1){
    success = Assign15d(x, Asin(x.number));
  }else{
    success = false;
  }

  return success;
}
function Acos15d(x){
  var success;

  if(x.number >=  -1 && x.number <= 1){
    success = Assign15d(x, Acos(x.number));
  }else{
    success = false;
  }

  return success;
}
function Atan15d(x){
  return Assign15d(x, Atan(x.number));
}
function Atan2_15d(a, y, x){
  return Assign15d(a, Atan2(y.number, x.number));
}
function Squareroot15d(x){
  var success;

  if(x.number >= 0){
    success = Assign15d(x, Math.sqrt(x.number));
  }else{
    success = false;
  }

  return success;
}
function Exp15d(x){
  return Assign15d(x, Exp(x.number));
}
function DivisibleBy15d(a, b){
  return ((a.number%b.number) == 0);
}
function Combinations15d(x, n, k){
  var success;

  if(IsInteger(n.number) && IsInteger(k.number)){
    if(n.number >= 1 && k.number >= 0 && n.number >= k.number){
      success = Assign15d(x, Combinations(n.number, k.number));
    }else{
      success = false;
    }
  }else{
    success = false;
  }

  return success;
}
function Permutations15d(x, n, k){
  var success;

  if(IsInteger(n.number) && IsInteger(k.number)){
    if(n.number >= 1 && k.number >= 0 && n.number >= k.number){
      success = Assign15d(x, Permutations(n.number, k.number));
    }else{
      success = false;
    }
  }else{
    success = false;
  }

  return success;
}
function Equals15d(a, b){
  var p, an, bn;
  var equals;

  an = ToNumber15d(a);
  bn = ToNumber15d(b);

  p = Math.max(a.digitsAfterDecimalPoint, b.digitsAfterDecimalPoint);

  equals = EpsilonCompare(an, bn, Math.pow(10,  -p));

  return equals;
}
function GreaterThan15d(a, b){
  var an, bn;

  an = ToNumber15d(a);
  bn = ToNumber15d(b);

  return an > bn;
}
function LessThan15d(a, b){
  var an, bn;

  an = ToNumber15d(a);
  bn = ToNumber15d(b);

  return an < bn;
}
function GreaterThanOrEqual15d(a, b){
  var an, bn;
  var equal;

  an = ToNumber15d(a);
  bn = ToNumber15d(b);

  equal = Equals15d(a, b);

  return an > bn || equal;
}
function LessThanOrEqual15d(a, b){
  var an, bn;
  var equal;

  an = ToNumber15d(a);
  bn = ToNumber15d(b);

  equal = Equals15d(a, b);

  return an < bn || equal;
}
function EpsilonCompare15d(a, b, epsilon){
  return EpsilonCompare(a.number, b.number, epsilon.number);
}
function GreatestCommonDivisor15d(x, a, b){
  var success;

  if(IsInteger(a.number) && IsInteger(b.number)){
    if(a.number >= 0 && b.number >= 0){
      success = Assign15d(x, GreatestCommonDivisor(a.number, b.number));
    }else{
      success = false;
    }
  }else{
    success = false;
  }

  return success;
}
function GCDWithSubtraction15d(x, a, b){
  var success;

  if(IsInteger(a.number) && IsInteger(b.number)){
    if(a.number >= 0 && b.number >= 0){
      success = Assign15d(x, GCDWithSubtraction(a.number, b.number));
    }else{
      success = false;
    }
  }else{
    success = false;
  }

  return success;
}
function IsInteger15d(a){
  return IsInteger(a.number);
}
function LeastCommonMultiple15d(x, a, b){
  var success;

  if(IsInteger(a.number) && IsInteger(b.number)){
    if(a.number != 0 && b.number != 0){
      success = Assign15d(x, LeastCommonMultiple(a.number, b.number));
    }else{
      success = false;
    }
  }else{
    success = false;
  }

  return success;
}
function Sign15d(a){
  return Sign(a.number);
}
function Max15d(x, a, b){
  return Assign15d(x, Max(a.number, b.number));
}
function Min15d(x, a, b){
  return Assign15d(x, Min(a.number, b.number));
}
function Power15d(x, a, b){
  var success;

  if(a.number != 0 || b.number != 0){
    if( !(a.number < 0 &&  !IsInteger(b.number) ) ){
      success = Assign15d(x, Power(a.number, b.number));
    }else{
      success = false;
    }
  }else{
    success = false;
  }

  return success;
}
function FormatToString15d(fp, digitsAfter){
  var result;

  result = FormatToStringWithSymbols15d(fp, digitsAfter, "".split(''), ".".split(''));

  return result;
}
function FormatToStringWithSymbols15d(fp, digitsAfter, thousandsSeparator, decimalPoint){
  var string;
  var i, j, p, d, t, sign, extra, decimal, digits, digitsBefore, thousandsChars, thousandsTimes, decimalPointChars;
  var characterReference;

  characterReference = {};

  decimal = Round(fp.number*Math.pow(10, digitsAfter));

  sign = 0;
  if(decimal < 0){
    sign = 1;
    decimal =  -decimal;
  }

  if(decimal != 0){
    digits = Math.floor(Math.log10(decimal) + 1);
  }else{
    digits = 1;
  }
  digitsBefore = digits - digitsAfter;

  if(digitsBefore <= 0){
    digitsBefore = 0;
    thousandsTimes = 0;
    digits = digitsAfter + 1;
  }else{
    thousandsTimes = Math.floor((digitsBefore - 1)/3);
  }
  thousandsChars = thousandsTimes*thousandsSeparator.length;

  if(digitsAfter == 0){
    decimalPointChars = 0;
  }else{
    decimalPointChars = decimalPoint.length;
  }

  string = [];
  string.length = sign + digits + thousandsChars + decimalPointChars;
  p = 0;

  if(sign > 0){
    string[p] = '-';
    p = p + 1;
  }

  for(i = 0; i < digits; i = i + 1){
    if(i == digitsBefore){
      if(i == 0){
        string[p] = '0';
        p = p + 1;
        digits = digits - 1;
      }

      for(j = 0; j < decimalPoint.length; j = j + 1){
        string[p] = decimalPoint[j];
        p = p + 1;
      }
    }

    if(i < digitsBefore){
      if((digitsBefore - i)%3 == 0 && i != 0){
        for(j = 0; j < thousandsSeparator.length; j = j + 1){
          string[p] = thousandsSeparator[j];
          p = p + 1;
        }
      }
    }

    d = Math.floor(decimal/Math.pow(10, digits - i - 1));
    d = d%10;

    nGetSingleDigitCharacterFromNumberWithCheck(d, 10, characterReference);
    string[p] = characterReference.characterValue;

    p = p + 1;
  }

  /* System.out.println(new String(string)); */
  return string;
}
function NumberToHumanReadable(n, digitsAfter, thousandsSeparator, decimalPoint){
  var str;
  var u;
  var d, p3;

  if(Math.abs(n) < 1){
    str = nCreateStringDecimalFromNumber(n);
  }else{
    d = Math.log10(n);

    p3 = Math.min(Math.floor(d/3), 8);

    if(p3 == 0){
      u = 'B';
    }else if(p3 == 1){
      u = 'K';
    }else if(p3 == 2){
      u = 'M';
    }else if(p3 == 3){
      u = 'G';
    }else if(p3 == 4){
      u = 'T';
    }else if(p3 == 5){
      u = 'P';
    }else if(p3 == 6){
      u = 'E';
    }else if(p3 == 7){
      u = 'Z';
    }else{
      u = 'Y';
    }

    if(p3 > 1){
      n = n/Math.pow(10, p3*3);
    }

    str = FormatToStringWithSymbols15d(Number15d(n), digitsAfter, thousandsSeparator, decimalPoint);

    if(p3 > 1){
      str = AppendCharacter(str, u);
    }
  }

  return str;
}
function NumberToHumanReadableBinaryPrefix(n, digitsAfter, thousandsSeparator, decimalPoint){
  var str;
  var u;
  var d, p3;

  if(Math.abs(n) < 1){
    str = nCreateStringDecimalFromNumber(n);
  }else{
    d = Math.floor(Math.log(n)/Math.log(2)) + 1;

    p3 = Math.min(Math.floor(d/10), 8);

    if(p3 == 0){
      u = "B".split('');
    }else if(p3 == 1){
      u = "Ki".split('');
    }else if(p3 == 2){
      u = "Mi".split('');
    }else if(p3 == 3){
      u = "Gi".split('');
    }else if(p3 == 4){
      u = "Ti".split('');
    }else if(p3 == 5){
      u = "Pi".split('');
    }else if(p3 == 6){
      u = "Ei".split('');
    }else if(p3 == 7){
      u = "Zi".split('');
    }else{
      u = "Yi".split('');
    }

    if(p3 > 1){
      n = n/Math.pow(2, p3*10);
    }

    str = FormatToStringWithSymbols15d(Number15d(n), digitsAfter, thousandsSeparator, decimalPoint);

    if(p3 > 1){
      str = AppendString(str, u);
    }
  }

  return str;
}
function WriteStringToStingStream(stream, index, src){
  var i;

  for(i = 0; i < src.length; i = i + 1){
    stream[index.numberValue + i] = src[i];
  }
  index.numberValue = index.numberValue + src.length;
}
function WriteCharacterToStingStream(stream, index, src){
  stream[index.numberValue] = src;
  index.numberValue = index.numberValue + 1;
}
function WriteBooleanToStingStream(stream, index, src){
  if(src){
    WriteStringToStingStream(stream, index, "true".split(''));
  }else{
    WriteStringToStingStream(stream, index, "false".split(''));
  }
}
function SubstringWithCheck(string, from, to, stringReference){
  var success;

  if(from >= 0 && from <= string.length && to >= 0 && to <= string.length && from <= to){
    stringReference.string = Substring(string, from, to);
    success = true;
  }else{
    success = false;
  }

  return success;
}
function Substring(string, from, to){
  var n;
  var i, lengthx;

  lengthx = to - from;

  n = [];
  n.length = lengthx;

  for(i = from; i < to; i = i + 1){
    n[i - from] = string[i];
  }

  return n;
}
function AppendString(s1, s2){
  var newString;

  newString = ConcatenateString(s1, s2);

  delete(s1);

  return newString;
}
function ConcatenateString(s1, s2){
  var newString;
  var i;

  newString = [];
  newString.length = s1.length + s2.length;

  for(i = 0; i < s1.length; i = i + 1){
    newString[i] = s1[i];
  }

  for(i = 0; i < s2.length; i = i + 1){
    newString[s1.length + i] = s2[i];
  }

  return newString;
}
function AppendCharacter(string, c){
  var newString;

  newString = ConcatenateCharacter(string, c);

  delete(string);

  return newString;
}
function ConcatenateCharacter(string, c){
  var newString;
  var i;
  newString = [];
  newString.length = string.length + 1;

  for(i = 0; i < string.length; i = i + 1){
    newString[i] = string[i];
  }

  newString[string.length] = c;

  return newString;
}
function SplitByCharacter(toSplit, splitBy){
  var split;
  var stringToSplitBy;

  stringToSplitBy = [];
  stringToSplitBy.length = 1;
  stringToSplitBy[0] = splitBy;

  split = SplitByString(toSplit, stringToSplitBy);

  delete(stringToSplitBy);

  return split;
}
function IndexOfCharacter(string, character, indexReference){
  var i;
  var found;

  found = false;
  for(i = 0; i < string.length &&  !found ; i = i + 1){
    if(string[i] == character){
      found = true;
      indexReference.numberValue = i;
    }
  }

  return found;
}
function SubstringEqualsWithCheck(string, from, substring, equalsReference){
  var success;

  if(from < string.length){
    success = true;
    equalsReference.booleanValue = SubstringEquals(string, from, substring);
  }else{
    success = false;
  }

  return success;
}
function SubstringEquals(string, from, substring){
  var i;
  var equal;

  equal = true;
  if(string.length - from >= substring.length){
    for(i = 0; i < substring.length && equal; i = i + 1){
      if(string[from + i] != substring[i]){
        equal = false;
      }
    }
  }else{
    equal = false;
  }

  return equal;
}
function IndexOfString(string, substring, indexReference){
  var i;
  var found;

  found = false;
  for(i = 0; i < string.length - substring.length + 1 &&  !found ; i = i + 1){
    if(SubstringEquals(string, i, substring)){
      found = true;
      indexReference.numberValue = i;
    }
  }

  return found;
}
function ContainsCharacter(string, character){
  var i;
  var found;

  found = false;
  for(i = 0; i < string.length &&  !found ; i = i + 1){
    if(string[i] == character){
      found = true;
    }
  }

  return found;
}
function ContainsString(string, substring){
  return IndexOfString(string, substring, {});
}
function ToUpperCase(string){
  var i;

  for(i = 0; i < string.length; i = i + 1){
    string[i] = charToUpperCase(string[i]);
  }
}
function ToLowerCase(string){
  var i;

  for(i = 0; i < string.length; i = i + 1){
    string[i] = charToLowerCase(string[i]);
  }
}
function EqualsIgnoreCase(a, b){
  var equal;
  var i;

  if(a.length == b.length){
    equal = true;
    for(i = 0; i < a.length && equal; i = i + 1){
      if(charToLowerCase(a[i]) != charToLowerCase(b[i])){
        equal = false;
      }
    }
  }else{
    equal = false;
  }

  return equal;
}
function ReplaceString(string, toReplace, replaceWith){
  var result;
  var i, j;
  var equalsReference;
  var success;
  var da;

  da = CreateDynamicArrayCharacters();

  equalsReference = {};

  for(i = 0; i < string.length; ){
    success = SubstringEqualsWithCheck(string, i, toReplace, equalsReference);
    if(success){
      success = equalsReference.booleanValue;
    }

    if(success && toReplace.length > 0){
      for(j = 0; j < replaceWith.length; j = j + 1){
        DynamicArrayAddCharacter(da, replaceWith[j]);
      }
      i = i + toReplace.length;
    }else{
      DynamicArrayAddCharacter(da, string[i]);
      i = i + 1;
    }
  }

  result = DynamicArrayCharactersToArray(da);

  FreeDynamicArrayCharacters(da);

  return result;
}
function ReplaceCharacterToNew(string, toReplace, replaceWith){
  var result;
  var i;

  result = [];
  result.length = string.length;

  for(i = 0; i < string.length; i = i + 1){
    if(string[i] == toReplace){
      result[i] = replaceWith;
    }else{
      result[i] = string[i];
    }
  }

  return result;
}
function ReplaceCharacter(string, toReplace, replaceWith){
  var i;

  for(i = 0; i < string.length; i = i + 1){
    if(string[i] == toReplace){
      string[i] = replaceWith;
    }
  }
}
function Trim(string){
  var result;
  var i, lastWhitespaceLocationStart, lastWhitespaceLocationEnd;
  var firstNonWhitespaceFound;

  /* Find whitepaces at the start. */
  lastWhitespaceLocationStart =  -1;
  firstNonWhitespaceFound = false;
  for(i = 0; i < string.length &&  !firstNonWhitespaceFound ; i = i + 1){
    if(charIsWhiteSpace(string[i])){
      lastWhitespaceLocationStart = i;
    }else{
      firstNonWhitespaceFound = true;
    }
  }

  /* Find whitepaces at the end. */
  lastWhitespaceLocationEnd = string.length;
  firstNonWhitespaceFound = false;
  for(i = string.length - 1; i >= 0 &&  !firstNonWhitespaceFound ; i = i - 1){
    if(charIsWhiteSpace(string[i])){
      lastWhitespaceLocationEnd = i;
    }else{
      firstNonWhitespaceFound = true;
    }
  }

  if(lastWhitespaceLocationStart < lastWhitespaceLocationEnd){
    result = Substring(string, lastWhitespaceLocationStart + 1, lastWhitespaceLocationEnd);
  }else{
    result = [];
    result.length = 0;
  }

  return result;
}
function StartsWith(string, start){
  var startsWithString;

  startsWithString = false;
  if(string.length >= start.length){
    startsWithString = SubstringEquals(string, 0, start);
  }

  return startsWithString;
}
function EndsWith(string, end){
  var endsWithString;

  endsWithString = false;
  if(string.length >= end.length){
    endsWithString = SubstringEquals(string, string.length - end.length, end);
  }

  return endsWithString;
}
function SplitByString(toSplit, splitBy){
  var split;
  var next;
  var i;
  var c;
  var n;

  split = [];
  split.length = 0;

  next = [];
  next.length = 0;
  for(i = 0; i < toSplit.length; ){
    c = toSplit[i];

    if(SubstringEquals(toSplit, i, splitBy)){
      n = {};
      n.string = next;
      split = AddString(split, n);
      next = [];
      next.length = 0;
      i = i + splitBy.length;
    }else{
      next = AppendCharacter(next, c);
      i = i + 1;
    }
  }

  n = {};
  n.string = next;
  split = AddString(split, n);

  return split;
}
function StringIsBefore(a, b){
  var before, equal, done;
  var i;

  before = false;
  equal = true;
  done = false;

  if(a.length == 0 && b.length > 0){
    before = true;
  }else{
    for(i = 0; i < a.length && i < b.length &&  !done ; i = i + 1){
      if(a[i] != b[i]){
        equal = false;
      }
      if(charCharacterIsBefore(a[i], b[i])){
        before = true;
      }
      if(charCharacterIsBefore(b[i], a[i])){
        done = true;
      }
    }

    if(equal){
      if(a.length < b.length){
        before = true;
      }
    }
  }

  return before;
}
function AssertFalse(b, failures){
  if(b){
    failures.numberValue = failures.numberValue + 1;
  }
}
function AssertTrue(b, failures){
  if( !b ){
    failures.numberValue = failures.numberValue + 1;
  }
}
function AssertEquals(a, b, failures){
  if(a != b){
    failures.numberValue = failures.numberValue + 1;
  }
}
function AssertBooleansEqual(a, b, failures){
  if(a != b){
    failures.numberValue = failures.numberValue + 1;
  }
}
function AssertCharactersEqual(a, b, failures){
  if(a != b){
    failures.numberValue = failures.numberValue + 1;
  }
}
function AssertStringEquals(a, b, failures){
  if( !arraysStringsEqual(a, b) ){
    failures.numberValue = failures.numberValue + 1;
  }
}
function AssertNumberArraysEqual(a, b, failures){
  var i;

  if(a.length == b.length){
    for(i = 0; i < a.length; i = i + 1){
      AssertEquals(a[i], b[i], failures);
    }
  }else{
    failures.numberValue = failures.numberValue + 1;
  }
}
function AssertBooleanArraysEqual(a, b, failures){
  var i;

  if(a.length == b.length){
    for(i = 0; i < a.length; i = i + 1){
      AssertBooleansEqual(a[i], b[i], failures);
    }
  }else{
    failures.numberValue = failures.numberValue + 1;
  }
}
function AssertStringArraysEqual(a, b, failures){
  var i;

  if(a.length == b.length){
    for(i = 0; i < a.length; i = i + 1){
      AssertStringEquals(a[i].string, b[i].string, failures);
    }
  }else{
    failures.numberValue = failures.numberValue + 1;
  }
}
function Negate(x){
  return  -x;
}
function Positive(x){
  return  +x;
}
function Factorial(x){
  var i, f;

  f = 1;

  for(i = 2; i <= x; i = i + 1){
    f = f*i;
  }

  return f;
}
function Round(x){
  return Math.floor(x + 0.5);
}
function BankersRound(x){
  var r;

  if(Absolute(x - Truncate(x)) == 0.5){
    if( !DivisibleBy(Round(x), 2) ){
      r = Round(x) - 1;
    }else{
      r = Round(x);
    }
  }else{
    r = Round(x);
  }

  return r;
}
function Ceil(x){
  return Math.ceil(x);
}
function Floor(x){
  return Math.floor(x);
}
function Truncate(x){
  var t;

  if(x >= 0){
    t = Math.floor(x);
  }else{
    t = Math.ceil(x);
  }

  return t;
}
function Absolute(x){
  return Math.abs(x);
}
function Logarithm(x){
  return Math.log10(x);
}
function NaturalLogarithm(x){
  return Math.log(x);
}
function Sin(x){
  return Math.sin(x);
}
function Cos(x){
  return Math.cos(x);
}
function Tan(x){
  return Math.tan(x);
}
function Asin(x){
  return Math.asin(x);
}
function Acos(x){
  return Math.acos(x);
}
function Atan(x){
  return Math.atan(x);
}
function Atan2(y, x){
  var a;

  /* Atan2 is an invalid operation when x = 0 and y = 0, but this method does not return errors. */
  a = 0;

  if(x > 0){
    a = Atan(y/x);
  }else if(x < 0 && y >= 0){
    a = Atan(y/x) + Math.PI;
  }else if(x < 0 && y < 0){
    a = Atan(y/x) - Math.PI;
  }else if(x == 0 && y > 0){
    a = Math.PI/2;
  }else if(x == 0 && y < 0){
    a =  -Math.PI/2;
  }

  return a;
}
function Squareroot(x){
  return Math.sqrt(x);
}
function Exp(x){
  return Math.exp(x);
}
function DivisibleBy(a, b){
  return ((a%b) == 0);
}
function Combinations(n, k){
  var i, j, c;

  c = 1;
  j = 1;
  i = n - k + 1;

  for(; i <= n; ){
    c = c*i;
    c = c/j;

    i = i + 1;
    j = j + 1;
  }

  return c;
}
function Permutations(n, k){
  var i, c;

  c = 1;

  for(i = n - k + 1; i <= n; i = i + 1){
    c = c*i;
  }

  return c;
}
function EpsilonCompare(a, b, epsilon){
  return Math.abs(a - b) < epsilon;
}
function GreatestCommonDivisor(a, b){
  var t;

  for(; b != 0; ){
    t = b;
    b = a%b;
    a = t;
  }

  return a;
}
function GCDWithSubtraction(a, b){
  var g;

  if(a == 0){
    g = b;
  }else{
    for(; b != 0; ){
      if(a > b){
        a = a - b;
      }else{
        b = b - a;
      }
    }

    g = a;
  }

  return g;
}
function IsInteger(a){
  return (a - Math.floor(a)) == 0;
}
function GreatestCommonDivisorWithCheck(a, b, gcdReference){
  var success;
  var gcd;

  if(IsInteger(a) && IsInteger(b)){
    gcd = GreatestCommonDivisor(a, b);
    gcdReference.numberValue = gcd;
    success = true;
  }else{
    success = false;
  }

  return success;
}
function LeastCommonMultiple(a, b){
  var lcm;

  if(a > 0 && b > 0){
    lcm = Math.abs(a*b)/GreatestCommonDivisor(a, b);
  }else{
    lcm = 0;
  }

  return lcm;
}
function Sign(a){
  var s;

  if(a > 0){
    s = 1;
  }else if(a < 0){
    s =  -1;
  }else{
    s = 0;
  }

  return s;
}
function Max(a, b){
  return Math.max(a, b);
}
function Min(a, b){
  return Math.min(a, b);
}
function Power(a, b){
  return Math.pow(a, b);
}
function Gamma(x){
  return LanczosApproximation(x);
}
function LogGamma(x){
  return Math.log(Gamma(x));
}
function LanczosApproximation(z){
  var p;
  var i, y, t, x;

  p = [];
  p.length = 8;
  p[0] = 676.5203681218851;
  p[1] =  -1259.1392167224028;
  p[2] = 771.32342877765313;
  p[3] =  -176.61502916214059;
  p[4] = 12.507343278686905;
  p[5] =  -0.13857109526572012;
  p[6] = 9.9843695780195716e-6;
  p[7] = 1.5056327351493116e-7;

  if(z < 0.5){
    y = Math.PI/(Math.sin(Math.PI*z)*LanczosApproximation(1 - z));
  }else{
    z = z - 1;
    x = 0.99999999999980993;
    for(i = 0; i < p.length; i = i + 1){
      x = x + p[i]/(z + i + 1);
    }
    t = z + p.length - 0.5;
    y = Math.sqrt(2*Math.PI)*Math.pow(t, z + 0.5)*Math.exp( -t)*x;
  }

  return y;
}
function Beta(x, y){
  return Gamma(x)*Gamma(y)/Gamma(x + y);
}
function Sinh(x){
  return (Math.exp(x) - Math.exp( -x))/2;
}
function Cosh(x){
  return (Math.exp(x) + Math.exp( -x))/2;
}
function Tanh(x){
  return Sinh(x)/Cosh(x);
}
function Cot(x){
  return 1/Math.tan(x);
}
function Sec(x){
  return 1/Math.cos(x);
}
function Csc(x){
  return 1/Math.sin(x);
}
function Coth(x){
  return Cosh(x)/Sinh(x);
}
function Sech(x){
  return 1/Cosh(x);
}
function Csch(x){
  return 1/Sinh(x);
}
function Error(x){
  var y, t, tau, c1, c2, c3, c4, c5, c6, c7, c8, c9, c10;

  if(x == 0){
    y = 0;
  }else if(x < 0){
    y =  -Error( -x);
  }else{
    c1 =  -1.26551223;
    c2 =  +1.00002368;
    c3 =  +0.37409196;
    c4 =  +0.09678418;
    c5 =  -0.18628806;
    c6 =  +0.27886807;
    c7 =  -1.13520398;
    c8 =  +1.48851587;
    c9 =  -0.82215223;
    c10 =  +0.17087277;

    t = 1/(1 + 0.5*Math.abs(x));

    tau = t*Math.exp( -(Math.pow(x, 2)) + c1 + t*(c2 + t*(c3 + t*(c4 + t*(c5 + t*(c6 + t*(c7 + t*(c8 + t*(c9 + t*c10)))))))));

    y = 1 - tau;
  }

  return y;
}
function ErrorInverse(x){
  var y, a, t;

  a = (8*(Math.PI - 3))/(3*Math.PI*(4 - Math.PI));

  t = 2/(Math.PI*a) + Math.log(1 - Math.pow(x, 2))/2;
  y = Sign(x)*Math.sqrt(Math.sqrt(Math.pow(t, 2) - Math.log(1 - Math.pow(x, 2))/a) - t);

  return y;
}
function FallingFactorial(x, n){
  var k, y;

  y = 1;

  for(k = 0; k <= n - 1; k = k + 1){
    y = y*(x - k);
  }

  return y;
}
function RisingFactorial(x, n){
  var k, y;

  y = 1;

  for(k = 0; k <= n - 1; k = k + 1){
    y = y*(x + k);
  }

  return y;
}
function Hypergeometric(a, b, c, z, maxIterations, precision){
  var y;

  if(Math.abs(z) >= 0.5){
    y = Math.pow(1 - z,  -a)*HypergeometricDirect(a, c - b, c, z/(z - 1), maxIterations, precision);
  }else{
    y = HypergeometricDirect(a, b, c, z, maxIterations, precision);
  }

  return y;
}
function HypergeometricDirect(a, b, c, z, maxIterations, precision){
  var y, yp, n;
  var done;

  y = 0;
  done = false;

  for(n = 0; n < maxIterations &&  !done ; n = n + 1){
    yp = RisingFactorial(a, n)*RisingFactorial(b, n)/RisingFactorial(c, n)*Math.pow(z, n)/Factorial(n);
    if(Math.abs(yp) < precision){
      done = true;
    }
    y = y + yp;
  }

  return y;
}
function BernouilliNumber(n){
  return AkiyamaTanigawaAlgorithm(n);
}
function AkiyamaTanigawaAlgorithm(n){
  var m, j, B;
  var A;

  A = [];
  A.length = n + 1;

  for(m = 0; m <= n; m = m + 1){
    A[m] = 1/(m + 1);
    for(j = m; j >= 1; j = j - 1){
      A[j - 1] = j*(A[j - 1] - A[j]);
    }
  }

  B = A[0];

  delete(A);

  return B;
}
function AddNumber(list, a){
  var newlist;
  var i;

  newlist = [];
  newlist.length = list.length + 1;
  for(i = 0; i < list.length; i = i + 1){
    newlist[i] = list[i];
  }
  newlist[list.length] = a;
		
  delete(list);
		
  return newlist;
}
function AddNumberRef(list, i){
  list.numberArray = AddNumber(list.numberArray, i);
}
function RemoveNumber(list, n){
  var newlist;
  var i;

  newlist = [];
  newlist.length = list.length - 1;

  if(n >= 0 && n < list.length){
    for(i = 0; i < list.length; i = i + 1){
      if(i < n){
        newlist[i] = list[i];
      }
      if(i > n){
        newlist[i - 1] = list[i];
      }
    }

    delete(list);
  }else{
    delete(newlist);
  }
		
  return newlist;
}
function GetNumberRef(list, i){
  return list.numberArray[i];
}
function RemoveNumberRef(list, i){
  list.numberArray = RemoveNumber(list.numberArray, i);
}
function AddString(list, a){
  var newlist;
  var i;

  newlist = [];
  newlist.length = list.length + 1;

  for(i = 0; i < list.length; i = i + 1){
    newlist[i] = list[i];
  }
  newlist[list.length] = a;
		
  delete(list);
		
  return newlist;
}
function AddStringRef(list, i){
  list.stringArray = AddString(list.stringArray, i);
}
function RemoveString(list, n){
  var newlist;
  var i;

  newlist = [];
  newlist.length = list.length - 1;

  if(n >= 0 && n < list.length){
    for(i = 0; i < list.length; i = i + 1){
      if(i < n){
        newlist[i] = list[i];
      }
      if(i > n){
        newlist[i - 1] = list[i];
      }
    }

    delete(list);
  }else{
    delete(newlist);
  }
		
  return newlist;
}
function GetStringRef(list, i){
  return list.stringArray[i];
}
function RemoveStringRef(list, i){
  list.stringArray = RemoveString(list.stringArray, i);
}
function CreateDynamicArrayCharacters(){
  var da;

  da = {};
  da.array = [];
  da.array.length = 10;
  da.lengthx = 0;

  return da;
}
function CreateDynamicArrayCharactersWithInitialCapacity(capacity){
  var da;

  da = {};
  da.array = [];
  da.array.length = capacity;
  da.lengthx = 0;

  return da;
}
function DynamicArrayAddCharacter(da, value){
  if(da.lengthx == da.array.length){
    DynamicArrayCharactersIncreaseSize(da);
  }

  da.array[da.lengthx] = value;
  da.lengthx = da.lengthx + 1;
}
function DynamicArrayCharactersIncreaseSize(da){
  var newLength, i;
  var newArray;

  newLength = Math.round(da.array.length*3/2);
  newArray = [];
  newArray.length = newLength;

  for(i = 0; i < da.array.length; i = i + 1){
    newArray[i] = da.array[i];
  }

  delete(da.array);

  da.array = newArray;
}
function DynamicArrayCharactersDecreaseSizeNecessary(da){
  var needsDecrease;

  needsDecrease = false;

  if(da.lengthx > 10){
    needsDecrease = da.lengthx <= Math.round(da.array.length*2/3);
  }

  return needsDecrease;
}
function DynamicArrayCharactersDecreaseSize(da){
  var newLength, i;
  var newArray;

  newLength = Math.round(da.array.length*2/3);
  newArray = [];
  newArray.length = newLength;

  for(i = 0; i < newLength; i = i + 1){
    newArray[i] = da.array[i];
  }

  delete(da.array);

  da.array = newArray;
}
function DynamicArrayCharactersIndex(da, index){
  return da.array[index];
}
function DynamicArrayCharactersLength(da){
  return da.lengthx;
}
function DynamicArrayInsertCharacter(da, index, value){
  var i;

  if(da.lengthx == da.array.length){
    DynamicArrayCharactersIncreaseSize(da);
  }

  for(i = da.lengthx; i > index; i = i - 1){
    da.array[i] = da.array[i - 1];
  }

  da.array[index] = value;

  da.lengthx = da.lengthx + 1;
}
function DynamicArrayCharacterSet(da, index, value){
  var success;

  if(index < da.lengthx){
    da.array[index] = value;
    success = true;
  }else{
    success = false;
  }

  return success;
}
function DynamicArrayRemoveCharacter(da, index){
  var i;

  for(i = index; i < da.lengthx - 1; i = i + 1){
    da.array[i] = da.array[i + 1];
  }

  da.lengthx = da.lengthx - 1;

  if(DynamicArrayCharactersDecreaseSizeNecessary(da)){
    DynamicArrayCharactersDecreaseSize(da);
  }
}
function FreeDynamicArrayCharacters(da){
  delete(da.array);
  delete(da);
}
function DynamicArrayCharactersToArray(da){
  var array;
  var i;

  array = [];
  array.length = da.lengthx;

  for(i = 0; i < da.lengthx; i = i + 1){
    array[i] = da.array[i];
  }

  return array;
}
function ArrayToDynamicArrayCharactersWithOptimalSize(array){
  var da;
  var i;
  var c, n, newCapacity;

  c = array.length;
  n = (Math.log(c) - 1)/Math.log(3/2);
  newCapacity = Math.floor(n) + 1;

  da = CreateDynamicArrayCharactersWithInitialCapacity(newCapacity);

  for(i = 0; i < array.length; i = i + 1){
    da.array[i] = array[i];
  }

  return da;
}
function ArrayToDynamicArrayCharacters(array){
  var da;

  da = {};
  da.array = arraysCopyString(array);
  da.lengthx = array.length;

  return da;
}
function DynamicArrayCharactersEqual(a, b){
  var equal;
  var i;

  equal = true;
  if(a.lengthx == b.lengthx){
    for(i = 0; i < a.lengthx && equal; i = i + 1){
      if(a.array[i] != b.array[i]){
        equal = false;
      }
    }
  }else{
    equal = false;
  }

  return equal;
}
function DynamicArrayCharactersToLinkedList(da){
  var ll;
  var i;

  ll = CreateLinkedListCharacter();

  for(i = 0; i < da.lengthx; i = i + 1){
    LinkedListAddCharacter(ll, da.array[i]);
  }

  return ll;
}
function LinkedListToDynamicArrayCharacters(ll){
  var da;
  var i;
  var node;

  node = ll.first;

  da = {};
  da.lengthx = LinkedListCharactersLength(ll);

  da.array = [];
  da.array.length = da.lengthx;

  for(i = 0; i < da.lengthx; i = i + 1){
    da.array[i] = node.value;
    node = node.next;
  }

  return da;
}
function AddBoolean(list, a){
  var newlist;
  var i;

  newlist = [];
  newlist.length = list.length + 1;
  for(i = 0; i < list.length; i = i + 1){
    newlist[i] = list[i];
  }
  newlist[list.length] = a;
		
  delete(list);
		
  return newlist;
}
function AddBooleanRef(list, i){
  list.booleanArray = AddBoolean(list.booleanArray, i);
}
function RemoveBoolean(list, n){
  var newlist;
  var i;

  newlist = [];
  newlist.length = list.length - 1;

  if(n >= 0 && n < list.length){
    for(i = 0; i < list.length; i = i + 1){
      if(i < n){
        newlist[i] = list[i];
      }
      if(i > n){
        newlist[i - 1] = list[i];
      }
    }

    delete(list);
  }else{
    delete(newlist);
  }
		
  return newlist;
}
function GetBooleanRef(list, i){
  return list.booleanArray[i];
}
function RemoveDecimalRef(list, i){
  list.booleanArray = RemoveBoolean(list.booleanArray, i);
}
function CreateLinkedListString(){
  var ll;

  ll = {};
  ll.first = {};
  ll.last = ll.first;
  ll.last.end = true;

  return ll;
}
function LinkedListAddString(ll, value){
  ll.last.end = false;
  ll.last.value = value;
  ll.last.next = {};
  ll.last.next.end = true;
  ll.last = ll.last.next;
}
function LinkedListStringsToArray(ll){
  var array;
  var lengthx, i;
  var node;

  node = ll.first;

  lengthx = LinkedListStringsLength(ll);

  array = [];
  array.length = lengthx;

  for(i = 0; i < lengthx; i = i + 1){
    array[i] = {};
    array[i].string = node.value;
    node = node.next;
  }

  return array;
}
function LinkedListStringsLength(ll){
  var l;
  var node;

  l = 0;
  node = ll.first;
  for(;  !node.end ; ){
    node = node.next;
    l = l + 1;
  }

  return l;
}
function FreeLinkedListString(ll){
  var node, prev;

  node = ll.first;

  for(;  !node.end ; ){
    prev = node;
    node = node.next;
    delete(prev);
  }

  delete(node);
}
function CreateLinkedListNumbers(){
  var ll;

  ll = {};
  ll.first = {};
  ll.last = ll.first;
  ll.last.end = true;

  return ll;
}
function CreateLinkedListNumbersArray(lengthx){
  var lls;
  var i;

  lls = [];
  lls.length = lengthx;
  for(i = 0; i < lls.length; i = i + 1){
    lls[i] = CreateLinkedListNumbers();
  }

  return lls;
}
function LinkedListAddNumber(ll, value){
  ll.last.end = false;
  ll.last.value = value;
  ll.last.next = {};
  ll.last.next.end = true;
  ll.last = ll.last.next;
}
function LinkedListNumbersLength(ll){
  var l;
  var node;

  l = 0;
  node = ll.first;
  for(;  !node.end ; ){
    node = node.next;
    l = l + 1;
  }

  return l;
}
function LinkedListNumbersIndex(ll, index){
  var i;
  var node;

  node = ll.first;
  for(i = 0; i < index; i = i + 1){
    node = node.next;
  }

  return node.value;
}
function LinkedListInsertNumber(ll, index, value){
  var i;
  var node, tmp;

  if(index == 0){
    tmp = ll.first;
    ll.first = {};
    ll.first.next = tmp;
    ll.first.value = value;
    ll.first.end = false;
  }else{
    node = ll.first;
    for(i = 0; i < index - 1; i = i + 1){
      node = node.next;
    }

    tmp = node.next;
    node.next = {};
    node.next.next = tmp;
    node.next.value = value;
    node.next.end = false;
  }
}
function LinkedListSet(ll, index, value){
  var i;
  var node;

  node = ll.first;
  for(i = 0; i < index; i = i + 1){
    node = node.next;
  }

  node.next.value = value;
}
function LinkedListRemoveNumber(ll, index){
  var i;
  var node, prev;

  node = ll.first;
  prev = ll.first;

  for(i = 0; i < index; i = i + 1){
    prev = node;
    node = node.next;
  }

  if(index == 0){
    ll.first = prev.next;
  }
  if( !prev.next.end ){
    prev.next = prev.next.next;
  }
}
function FreeLinkedListNumbers(ll){
  var node, prev;

  node = ll.first;

  for(;  !node.end ; ){
    prev = node;
    node = node.next;
    delete(prev);
  }

  delete(node);
}
function FreeLinkedListNumbersArray(lls){
  var i;

  for(i = 0; i < lls.length; i = i + 1){
    FreeLinkedListNumbers(lls[i]);
  }
  delete(lls);
}
function LinkedListNumbersToArray(ll){
  var array;
  var lengthx, i;
  var node;

  node = ll.first;

  lengthx = LinkedListNumbersLength(ll);

  array = [];
  array.length = lengthx;

  for(i = 0; i < lengthx; i = i + 1){
    array[i] = node.value;
    node = node.next;
  }

  return array;
}
function ArrayToLinkedListNumbers(array){
  var ll;
  var i;

  ll = CreateLinkedListNumbers();

  for(i = 0; i < array.length; i = i + 1){
    LinkedListAddNumber(ll, array[i]);
  }

  return ll;
}
function LinkedListNumbersEqual(a, b){
  var equal, done;
  var an, bn;

  an = a.first;
  bn = b.first;

  equal = true;
  done = false;
  for(; equal &&  !done ; ){
    if(an.end == bn.end){
      if(an.end){
        done = true;
      }else if(an.value == bn.value){
        an = an.next;
        bn = bn.next;
      }else{
        equal = false;
      }
    }else{
      equal = false;
    }
  }

  return equal;
}
function CreateLinkedListCharacter(){
  var ll;

  ll = {};
  ll.first = {};
  ll.last = ll.first;
  ll.last.end = true;

  return ll;
}
function LinkedListAddCharacter(ll, value){
  ll.last.end = false;
  ll.last.value = value;
  ll.last.next = {};
  ll.last.next.end = true;
  ll.last = ll.last.next;
}
function LinkedListCharactersToArray(ll){
  var array;
  var lengthx, i;
  var node;

  node = ll.first;

  lengthx = LinkedListCharactersLength(ll);

  array = [];
  array.length = lengthx;

  for(i = 0; i < lengthx; i = i + 1){
    array[i] = node.value;
    node = node.next;
  }

  return array;
}
function LinkedListCharactersLength(ll){
  var l;
  var node;

  l = 0;
  node = ll.first;
  for(;  !node.end ; ){
    node = node.next;
    l = l + 1;
  }

  return l;
}
function FreeLinkedListCharacter(ll){
  var node, prev;

  node = ll.first;

  for(;  !node.end ; ){
    prev = node;
    node = node.next;
    delete(prev);
  }

  delete(node);
}
function LinkedListCharactersAddString(ll, str){
  var i;

  for(i = 0; i < str.length; i = i + 1){
    LinkedListAddCharacter(ll, str[i]);
  }
}
function CreateDynamicArrayNumbers(){
  var da;

  da = {};
  da.array = [];
  da.array.length = 10;
  da.lengthx = 0;

  return da;
}
function CreateDynamicArrayNumbersWithInitialCapacity(capacity){
  var da;

  da = {};
  da.array = [];
  da.array.length = capacity;
  da.lengthx = 0;

  return da;
}
function DynamicArrayAddNumber(da, value){
  if(da.lengthx == da.array.length){
    DynamicArrayNumbersIncreaseSize(da);
  }

  da.array[da.lengthx] = value;
  da.lengthx = da.lengthx + 1;
}
function DynamicArrayNumbersIncreaseSize(da){
  var newLength, i;
  var newArray;

  newLength = Math.round(da.array.length*3/2);
  newArray = [];
  newArray.length = newLength;

  for(i = 0; i < da.array.length; i = i + 1){
    newArray[i] = da.array[i];
  }

  delete(da.array);

  da.array = newArray;
}
function DynamicArrayNumbersDecreaseSizeNecessary(da){
  var needsDecrease;

  needsDecrease = false;

  if(da.lengthx > 10){
    needsDecrease = da.lengthx <= Math.round(da.array.length*2/3);
  }

  return needsDecrease;
}
function DynamicArrayNumbersDecreaseSize(da){
  var newLength, i;
  var newArray;

  newLength = Math.round(da.array.length*2/3);
  newArray = [];
  newArray.length = newLength;

  for(i = 0; i < newLength; i = i + 1){
    newArray[i] = da.array[i];
  }

  delete(da.array);

  da.array = newArray;
}
function DynamicArrayNumbersIndex(da, index){
  return da.array[index];
}
function DynamicArrayNumbersLength(da){
  return da.lengthx;
}
function DynamicArrayInsertNumber(da, index, value){
  var i;

  if(da.lengthx == da.array.length){
    DynamicArrayNumbersIncreaseSize(da);
  }

  for(i = da.lengthx; i > index; i = i - 1){
    da.array[i] = da.array[i - 1];
  }

  da.array[index] = value;

  da.lengthx = da.lengthx + 1;
}
function DynamicArrayNumberSet(da, index, value){
  var success;

  if(index < da.lengthx){
    da.array[index] = value;
    success = true;
  }else{
    success = false;
  }

  return success;
}
function DynamicArrayRemoveNumber(da, index){
  var i;

  for(i = index; i < da.lengthx - 1; i = i + 1){
    da.array[i] = da.array[i + 1];
  }

  da.lengthx = da.lengthx - 1;

  if(DynamicArrayNumbersDecreaseSizeNecessary(da)){
    DynamicArrayNumbersDecreaseSize(da);
  }
}
function FreeDynamicArrayNumbers(da){
  delete(da.array);
  delete(da);
}
function DynamicArrayNumbersToArray(da){
  var array;
  var i;

  array = [];
  array.length = da.lengthx;

  for(i = 0; i < da.lengthx; i = i + 1){
    array[i] = da.array[i];
  }

  return array;
}
function ArrayToDynamicArrayNumbersWithOptimalSize(array){
  var da;
  var i;
  var c, n, newCapacity;

  /*
         c = 10*(3/2)^n
         log(c) = log(10*(3/2)^n)
         log(c) = log(10) + log((3/2)^n)
         log(c) = 1 + log((3/2)^n)
         log(c) - 1 = log((3/2)^n)
         log(c) - 1 = n*log(3/2)
         n = (log(c) - 1)/log(3/2)
         */
  c = array.length;
  n = (Math.log(c) - 1)/Math.log(3/2);
  newCapacity = Math.floor(n) + 1;

  da = CreateDynamicArrayNumbersWithInitialCapacity(newCapacity);

  for(i = 0; i < array.length; i = i + 1){
    da.array[i] = array[i];
  }

  return da;
}
function ArrayToDynamicArrayNumbers(array){
  var da;

  da = {};
  da.array = arraysCopyNumberArray(array);
  da.lengthx = array.length;

  return da;
}
function DynamicArrayNumbersEqual(a, b){
  var equal;
  var i;

  equal = true;
  if(a.lengthx == b.lengthx){
    for(i = 0; i < a.lengthx && equal; i = i + 1){
      if(a.array[i] != b.array[i]){
        equal = false;
      }
    }
  }else{
    equal = false;
  }

  return equal;
}
function DynamicArrayNumbersToLinkedList(da){
  var ll;
  var i;

  ll = CreateLinkedListNumbers();

  for(i = 0; i < da.lengthx; i = i + 1){
    LinkedListAddNumber(ll, da.array[i]);
  }

  return ll;
}
function LinkedListToDynamicArrayNumbers(ll){
  var da;
  var i;
  var node;

  node = ll.first;

  da = {};
  da.lengthx = LinkedListNumbersLength(ll);

  da.array = [];
  da.array.length = da.lengthx;

  for(i = 0; i < da.lengthx; i = i + 1){
    da.array[i] = node.value;
    node = node.next;
  }

  return da;
}
function DynamicArrayNumbersIndexOf(arr, n, foundReference){
  var found;
  var i;

  found = false;
  for(i = 0; i < arr.lengthx &&  !found ; i = i + 1){
    if(arr.array[i] == n){
      found = true;
    }
  }
  if( !found ){
    i =  -1;
  }else{
    i = i - 1;
  }

  foundReference.booleanValue = found;

  return i;
}
function DynamicArrayNumbersIsInArray(arr, n){
  var found;
  var i;

  found = false;
  for(i = 0; i < arr.lengthx &&  !found ; i = i + 1){
    if(arr.array[i] == n){
      found = true;
    }
  }

  return found;
}
function AddCharacter(list, a){
  var newlist;
  var i;

  newlist = [];
  newlist.length = list.length + 1;
  for(i = 0; i < list.length; i = i + 1){
    newlist[i] = list[i];
  }
  newlist[list.length] = a;
		
  delete(list);
		
  return newlist;
}
function AddCharacterRef(list, i){
  list.string = AddCharacter(list.string, i);
}
function RemoveCharacter(list, n){
  var newlist;
  var i;

  newlist = [];
  newlist.length = list.length - 1;

  if(n >= 0 && n < list.length){
    for(i = 0; i < list.length; i = i + 1){
      if(i < n){
        newlist[i] = list[i];
      }
      if(i > n){
        newlist[i - 1] = list[i];
      }
    }

    delete(list);
  }else{
    delete(newlist);
  }

  return newlist;
}
function GetCharacterRef(list, i){
  return list.string[i];
}
function RemoveCharacterRef(list, i){
  list.string = RemoveCharacter(list.string, i);
}
function charToLowerCase(character){
  var toReturn;

  toReturn = character;
  if(character == 'A'){
    toReturn = 'a';
  }else if(character == 'B'){
    toReturn = 'b';
  }else if(character == 'C'){
    toReturn = 'c';
  }else if(character == 'D'){
    toReturn = 'd';
  }else if(character == 'E'){
    toReturn = 'e';
  }else if(character == 'F'){
    toReturn = 'f';
  }else if(character == 'G'){
    toReturn = 'g';
  }else if(character == 'H'){
    toReturn = 'h';
  }else if(character == 'I'){
    toReturn = 'i';
  }else if(character == 'J'){
    toReturn = 'j';
  }else if(character == 'K'){
    toReturn = 'k';
  }else if(character == 'L'){
    toReturn = 'l';
  }else if(character == 'M'){
    toReturn = 'm';
  }else if(character == 'N'){
    toReturn = 'n';
  }else if(character == 'O'){
    toReturn = 'o';
  }else if(character == 'P'){
    toReturn = 'p';
  }else if(character == 'Q'){
    toReturn = 'q';
  }else if(character == 'R'){
    toReturn = 'r';
  }else if(character == 'S'){
    toReturn = 's';
  }else if(character == 'T'){
    toReturn = 't';
  }else if(character == 'U'){
    toReturn = 'u';
  }else if(character == 'V'){
    toReturn = 'v';
  }else if(character == 'W'){
    toReturn = 'w';
  }else if(character == 'X'){
    toReturn = 'x';
  }else if(character == 'Y'){
    toReturn = 'y';
  }else if(character == 'Z'){
    toReturn = 'z';
  }

  return toReturn;
}
function charToUpperCase(character){
  var toReturn;

  toReturn = character;
  if(character == 'a'){
    toReturn = 'A';
  }else if(character == 'b'){
    toReturn = 'B';
  }else if(character == 'c'){
    toReturn = 'C';
  }else if(character == 'd'){
    toReturn = 'D';
  }else if(character == 'e'){
    toReturn = 'E';
  }else if(character == 'f'){
    toReturn = 'F';
  }else if(character == 'g'){
    toReturn = 'G';
  }else if(character == 'h'){
    toReturn = 'H';
  }else if(character == 'i'){
    toReturn = 'I';
  }else if(character == 'j'){
    toReturn = 'J';
  }else if(character == 'k'){
    toReturn = 'K';
  }else if(character == 'l'){
    toReturn = 'L';
  }else if(character == 'm'){
    toReturn = 'M';
  }else if(character == 'n'){
    toReturn = 'N';
  }else if(character == 'o'){
    toReturn = 'O';
  }else if(character == 'p'){
    toReturn = 'P';
  }else if(character == 'q'){
    toReturn = 'Q';
  }else if(character == 'r'){
    toReturn = 'R';
  }else if(character == 's'){
    toReturn = 'S';
  }else if(character == 't'){
    toReturn = 'T';
  }else if(character == 'u'){
    toReturn = 'U';
  }else if(character == 'v'){
    toReturn = 'V';
  }else if(character == 'w'){
    toReturn = 'W';
  }else if(character == 'x'){
    toReturn = 'X';
  }else if(character == 'y'){
    toReturn = 'Y';
  }else if(character == 'z'){
    toReturn = 'Z';
  }

  return toReturn;
}
function charIsUpperCase(character){
  var isUpper;

  isUpper = true;
  if(character == 'A'){
  }else if(character == 'B'){
  }else if(character == 'C'){
  }else if(character == 'D'){
  }else if(character == 'E'){
  }else if(character == 'F'){
  }else if(character == 'G'){
  }else if(character == 'H'){
  }else if(character == 'I'){
  }else if(character == 'J'){
  }else if(character == 'K'){
  }else if(character == 'L'){
  }else if(character == 'M'){
  }else if(character == 'N'){
  }else if(character == 'O'){
  }else if(character == 'P'){
  }else if(character == 'Q'){
  }else if(character == 'R'){
  }else if(character == 'S'){
  }else if(character == 'T'){
  }else if(character == 'U'){
  }else if(character == 'V'){
  }else if(character == 'W'){
  }else if(character == 'X'){
  }else if(character == 'Y'){
  }else if(character == 'Z'){
  }else{
    isUpper = false;
  }

  return isUpper;
}
function charIsLowerCase(character){
  var isLower;

  isLower = true;
  if(character == 'a'){
  }else if(character == 'b'){
  }else if(character == 'c'){
  }else if(character == 'd'){
  }else if(character == 'e'){
  }else if(character == 'f'){
  }else if(character == 'g'){
  }else if(character == 'h'){
  }else if(character == 'i'){
  }else if(character == 'j'){
  }else if(character == 'k'){
  }else if(character == 'l'){
  }else if(character == 'm'){
  }else if(character == 'n'){
  }else if(character == 'o'){
  }else if(character == 'p'){
  }else if(character == 'q'){
  }else if(character == 'r'){
  }else if(character == 's'){
  }else if(character == 't'){
  }else if(character == 'u'){
  }else if(character == 'v'){
  }else if(character == 'w'){
  }else if(character == 'x'){
  }else if(character == 'y'){
  }else if(character == 'z'){
  }else{
    isLower = false;
  }

  return isLower;
}
function charIsLetter(character){
  return charIsUpperCase(character) || charIsLowerCase(character);
}
function charIsNumber(character){
  var isNumberx;

  isNumberx = true;
  if(character == '0'){
  }else if(character == '1'){
  }else if(character == '2'){
  }else if(character == '3'){
  }else if(character == '4'){
  }else if(character == '5'){
  }else if(character == '6'){
  }else if(character == '7'){
  }else if(character == '8'){
  }else if(character == '9'){
  }else{
    isNumberx = false;
  }

  return isNumberx;
}
function charIsWhiteSpace(character){
  var isWhiteSpacex;

  isWhiteSpacex = true;
  if(character == ' '){
  }else if(character == '\t'){
  }else if(character == '\n'){
  }else if(character == '\r'){
  }else{
    isWhiteSpacex = false;
  }

  return isWhiteSpacex;
}
function charIsSymbol(character){
  var isSymbolx;

  isSymbolx = true;
  if(character == '!'){
  }else if(character == '\"'){
  }else if(character == '#'){
  }else if(character == '$'){
  }else if(character == '%'){
  }else if(character == '&'){
  }else if(character == '\''){
  }else if(character == '('){
  }else if(character == ')'){
  }else if(character == '*'){
  }else if(character == '+'){
  }else if(character == ','){
  }else if(character == '-'){
  }else if(character == '.'){
  }else if(character == '/'){
  }else if(character == ':'){
  }else if(character == ';'){
  }else if(character == '<'){
  }else if(character == '='){
  }else if(character == '>'){
  }else if(character == '?'){
  }else if(character == '@'){
  }else if(character == '['){
  }else if(character == '\\'){
  }else if(character == ']'){
  }else if(character == '^'){
  }else if(character == '_'){
  }else if(character == '`'){
  }else if(character == '{'){
  }else if(character == '|'){
  }else if(character == '}'){
  }else if(character == '~'){
  }else{
    isSymbolx = false;
  }

  return isSymbolx;
}
function charCharacterIsBefore(a, b){
  var ad, bd;

  ad = a.charCodeAt(0);
  bd = b.charCodeAt(0);

  return ad < bd;
}
function charDecimalDigitToCharacter(digit){
  var c;
  if(digit == 1){
    c = '1';
  }else if(digit == 2){
    c = '2';
  }else if(digit == 3){
    c = '3';
  }else if(digit == 4){
    c = '4';
  }else if(digit == 5){
    c = '5';
  }else if(digit == 6){
    c = '6';
  }else if(digit == 7){
    c = '7';
  }else if(digit == 8){
    c = '8';
  }else if(digit == 9){
    c = '9';
  }else{
    c = '0';
  }
  return c;
}
function charCharacterToDecimalDigit(c){
  var digit;

  if(c == '1'){
    digit = 1;
  }else if(c == '2'){
    digit = 2;
  }else if(c == '3'){
    digit = 3;
  }else if(c == '4'){
    digit = 4;
  }else if(c == '5'){
    digit = 5;
  }else if(c == '6'){
    digit = 6;
  }else if(c == '7'){
    digit = 7;
  }else if(c == '8'){
    digit = 8;
  }else if(c == '9'){
    digit = 9;
  }else{
    digit = 0;
  }

  return digit;
}
function arraysStringToNumberArray(string){
  var i;
  var array;

  array = [];
  array.length = string.length;

  for(i = 0; i < string.length; i = i + 1){
    array[i] = string[i].charCodeAt(0);
  }
  return array;
}
function arraysNumberArrayToString(array){
  var i;
  var string;

  string = [];
  string.length = array.length;

  for(i = 0; i < array.length; i = i + 1){
    string[i] = String.fromCharCode(array[i]);
  }
  return string;
}
function arraysNumberArraysEqual(a, b){
  var equal;
  var i;

  equal = true;
  if(a.length == b.length){
    for(i = 0; i < a.length && equal; i = i + 1){
      if(a[i] != b[i]){
        equal = false;
      }
    }
  }else{
    equal = false;
  }

  return equal;
}
function arraysBooleanArraysEqual(a, b){
  var equal;
  var i;

  equal = true;
  if(a.length == b.length){
    for(i = 0; i < a.length && equal; i = i + 1){
      if(a[i] != b[i]){
        equal = false;
      }
    }
  }else{
    equal = false;
  }

  return equal;
}
function arraysStringsEqual(a, b){
  var equal;
  var i;

  equal = true;
  if(a.length == b.length){
    for(i = 0; i < a.length && equal; i = i + 1){
      if(a[i] != b[i]){
        equal = false;
      }
    }
  }else{
    equal = false;
  }

  return equal;
}
function arraysFillNumberArray(a, value){
  var i;

  for(i = 0; i < a.length; i = i + 1){
    a[i] = value;
  }
}
function arraysFillString(a, value){
  var i;

  for(i = 0; i < a.length; i = i + 1){
    a[i] = value;
  }
}
function arraysFillBooleanArray(a, value){
  var i;

  for(i = 0; i < a.length; i = i + 1){
    a[i] = value;
  }
}
function arraysFillNumberArrayRange(a, value, from, to){
  var i, lengthx;
  var success;

  if(from >= 0 && from <= a.length && to >= 0 && to <= a.length && from <= to){
    lengthx = to - from;
    for(i = 0; i < lengthx; i = i + 1){
      a[from + i] = value;
    }

    success = true;
  }else{
    success = false;
  }

  return success;
}
function arraysFillBooleanArrayRange(a, value, from, to){
  var i, lengthx;
  var success;

  if(from >= 0 && from <= a.length && to >= 0 && to <= a.length && from <= to){
    lengthx = to - from;
    for(i = 0; i < lengthx; i = i + 1){
      a[from + i] = value;
    }

    success = true;
  }else{
    success = false;
  }

  return success;
}
function arraysFillStringRange(a, value, from, to){
  var i, lengthx;
  var success;

  if(from >= 0 && from <= a.length && to >= 0 && to <= a.length && from <= to){
    lengthx = to - from;
    for(i = 0; i < lengthx; i = i + 1){
      a[from + i] = value;
    }

    success = true;
  }else{
    success = false;
  }

  return success;
}
function arraysCopyNumberArray(a){
  var i;
  var n;

  n = [];
  n.length = a.length;

  for(i = 0; i < a.length; i = i + 1){
    n[i] = a[i];
  }

  return n;
}
function arraysCopyBooleanArray(a){
  var i;
  var n;

  n = [];
  n.length = a.length;

  for(i = 0; i < a.length; i = i + 1){
    n[i] = a[i];
  }

  return n;
}
function arraysCopyString(a){
  var i;
  var n;

  n = [];
  n.length = a.length;

  for(i = 0; i < a.length; i = i + 1){
    n[i] = a[i];
  }

  return n;
}
function arraysCopyNumberArrayRange(a, from, to, copyReference){
  var i, lengthx;
  var n;
  var success;

  if(from >= 0 && from <= a.length && to >= 0 && to <= a.length && from <= to){
    lengthx = to - from;
    n = [];
    n.length = lengthx;

    for(i = 0; i < lengthx; i = i + 1){
      n[i] = a[from + i];
    }

    copyReference.numberArray = n;
    success = true;
  }else{
    success = false;
  }

  return success;
}
function arraysCopyBooleanArrayRange(a, from, to, copyReference){
  var i, lengthx;
  var n;
  var success;

  if(from >= 0 && from <= a.length && to >= 0 && to <= a.length && from <= to){
    lengthx = to - from;
    n = [];
    n.length = lengthx;

    for(i = 0; i < lengthx; i = i + 1){
      n[i] = a[from + i];
    }

    copyReference.booleanArray = n;
    success = true;
  }else{
    success = false;
  }

  return success;
}
function arraysCopyStringRange(a, from, to, copyReference){
  var i, lengthx;
  var n;
  var success;

  if(from >= 0 && from <= a.length && to >= 0 && to <= a.length && from <= to){
    lengthx = to - from;
    n = [];
    n.length = lengthx;

    for(i = 0; i < lengthx; i = i + 1){
      n[i] = a[from + i];
    }

    copyReference.string = n;
    success = true;
  }else{
    success = false;
  }

  return success;
}
function arraysIsLastElement(lengthx, index){
  return index + 1 == lengthx;
}
function arraysCreateNumberArray(lengthx, value){
  var array;

  array = [];
  array.length = lengthx;
  arraysFillNumberArray(array, value);

  return array;
}
function arraysCreateBooleanArray(lengthx, value){
  var array;

  array = [];
  array.length = lengthx;
  arraysFillBooleanArray(array, value);

  return array;
}
function arraysCreateString(lengthx, value){
  var array;

  array = [];
  array.length = lengthx;
  arraysFillString(array, value);

  return array;
}
function arraysSwapElementsOfNumberArray(A, ai, bi){
  var tmp;

  tmp = A[ai];
  A[ai] = A[bi];
  A[bi] = tmp;
}
function arraysSwapElementsOfStringArray(A, ai, bi){
  var tmp;

  tmp = A.stringArray[ai];
  A.stringArray[ai] = A.stringArray[bi];
  A.stringArray[bi] = tmp;
}
function arraysReverseNumberArray(array){
  var i;

  for(i = 0; i < array.length/2; i = i + 1){
    arraysSwapElementsOfNumberArray(array, i, array.length - i - 1);
  }
}


module.exports = {
	ParseORCGiro: ParseORCGiro,
	ParseORCGiroOppdrag: ParseORCGiroOppdrag,
	ParseORCGiroTransaksjon: ParseORCGiroTransaksjon,
	nCreateStringScientificNotationDecimalFromNumber: nCreateStringScientificNotationDecimalFromNumber,
	nCreateStringDecimalFromNumber: nCreateStringDecimalFromNumber,
	nCreateStringFromNumberWithCheck: nCreateStringFromNumberWithCheck,
	nGetMaximumDigitsForBase: nGetMaximumDigitsForBase,
	nGetFirstDigitPosition: nGetFirstDigitPosition,
	nGetSingleDigitCharacterFromNumberWithCheck: nGetSingleDigitCharacterFromNumberWithCheck,
	nGetDigitCharacterTable: nGetDigitCharacterTable,
	nCreateNumberFromDecimalStringWithCheck: nCreateNumberFromDecimalStringWithCheck,
	nCreateNumberFromDecimalString: nCreateNumberFromDecimalString,
	nCreateNumberFromStringWithCheck: nCreateNumberFromStringWithCheck,
	nCreateNumberFromParts: nCreateNumberFromParts,
	nExtractPartsFromNumberString: nExtractPartsFromNumberString,
	nGetNumberFromNumberCharacterForBase: nGetNumberFromNumberCharacterForBase,
	nCharacterIsNumberCharacterInBase: nCharacterIsNumberCharacterInBase,
	nStringToNumberArray: nStringToNumberArray,
	nStringToNumberArrayWithCheck: nStringToNumberArrayWithCheck,
	CreateBooleanReference: CreateBooleanReference,
	CreateBooleanArrayReference: CreateBooleanArrayReference,
	CreateBooleanArrayReferenceLengthValue: CreateBooleanArrayReferenceLengthValue,
	FreeBooleanArrayReference: FreeBooleanArrayReference,
	CreateCharacterReference: CreateCharacterReference,
	CreateNumberReference: CreateNumberReference,
	CreateNumberArrayReference: CreateNumberArrayReference,
	CreateNumberArrayReferenceLengthValue: CreateNumberArrayReferenceLengthValue,
	FreeNumberArrayReference: FreeNumberArrayReference,
	CreateStringReference: CreateStringReference,
	CreateStringReferenceLengthValue: CreateStringReferenceLengthValue,
	FreeStringReference: FreeStringReference,
	CreateStringArrayReference: CreateStringArrayReference,
	CreateStringArrayReferenceLengthValue: CreateStringArrayReferenceLengthValue,
	FreeStringArrayReference: FreeStringArrayReference,
	CreateFixedPoint30d: CreateFixedPoint30d,
	CreateFixedPoint15d: CreateFixedPoint15d,
	ToNumber15d: ToNumber15d,
	Number15d: Number15d,
	Assign15d: Assign15d,
	Assign15dFloor: Assign15dFloor,
	FixedPointIsValid15d: FixedPointIsValid15d,
	WillOverflow15d: WillOverflow15d,
	RoundToDigits: RoundToDigits,
	FloorToDigits: FloorToDigits,
	ToString15d: ToString15d,
	Add15d: Add15d,
	AddInto15d: AddInto15d,
	Subtract15d: Subtract15d,
	SubtractInto15d: SubtractInto15d,
	Multiply15d: Multiply15d,
	MultiplyInto15d: MultiplyInto15d,
	DivideFloored15d: DivideFloored15d,
	DivideFlooredInto15d: DivideFlooredInto15d,
	Negate15d: Negate15d,
	Positive15d: Positive15d,
	Factorial15d: Factorial15d,
	Round15d: Round15d,
	BankersRound15d: BankersRound15d,
	Ceil15d: Ceil15d,
	Floor15d: Floor15d,
	Truncate15d: Truncate15d,
	Absolute15d: Absolute15d,
	Logarithm15d: Logarithm15d,
	NaturalLogarithm15d: NaturalLogarithm15d,
	Sin15d: Sin15d,
	Cos15d: Cos15d,
	Tan15d: Tan15d,
	Asin15d: Asin15d,
	Acos15d: Acos15d,
	Atan15d: Atan15d,
	Atan2_15d: Atan2_15d,
	Squareroot15d: Squareroot15d,
	Exp15d: Exp15d,
	DivisibleBy15d: DivisibleBy15d,
	Combinations15d: Combinations15d,
	Permutations15d: Permutations15d,
	Equals15d: Equals15d,
	GreaterThan15d: GreaterThan15d,
	LessThan15d: LessThan15d,
	GreaterThanOrEqual15d: GreaterThanOrEqual15d,
	LessThanOrEqual15d: LessThanOrEqual15d,
	EpsilonCompare15d: EpsilonCompare15d,
	GreatestCommonDivisor15d: GreatestCommonDivisor15d,
	GCDWithSubtraction15d: GCDWithSubtraction15d,
	IsInteger15d: IsInteger15d,
	LeastCommonMultiple15d: LeastCommonMultiple15d,
	Sign15d: Sign15d,
	Max15d: Max15d,
	Min15d: Min15d,
	Power15d: Power15d,
	FormatToString15d: FormatToString15d,
	FormatToStringWithSymbols15d: FormatToStringWithSymbols15d,
	NumberToHumanReadable: NumberToHumanReadable,
	NumberToHumanReadableBinaryPrefix: NumberToHumanReadableBinaryPrefix,
	WriteStringToStingStream: WriteStringToStingStream,
	WriteCharacterToStingStream: WriteCharacterToStingStream,
	WriteBooleanToStingStream: WriteBooleanToStingStream,
	SubstringWithCheck: SubstringWithCheck,
	Substring: Substring,
	AppendString: AppendString,
	ConcatenateString: ConcatenateString,
	AppendCharacter: AppendCharacter,
	ConcatenateCharacter: ConcatenateCharacter,
	SplitByCharacter: SplitByCharacter,
	IndexOfCharacter: IndexOfCharacter,
	SubstringEqualsWithCheck: SubstringEqualsWithCheck,
	SubstringEquals: SubstringEquals,
	IndexOfString: IndexOfString,
	ContainsCharacter: ContainsCharacter,
	ContainsString: ContainsString,
	ToUpperCase: ToUpperCase,
	ToLowerCase: ToLowerCase,
	EqualsIgnoreCase: EqualsIgnoreCase,
	ReplaceString: ReplaceString,
	ReplaceCharacterToNew: ReplaceCharacterToNew,
	ReplaceCharacter: ReplaceCharacter,
	Trim: Trim,
	StartsWith: StartsWith,
	EndsWith: EndsWith,
	SplitByString: SplitByString,
	StringIsBefore: StringIsBefore,
	AssertFalse: AssertFalse,
	AssertTrue: AssertTrue,
	AssertEquals: AssertEquals,
	AssertBooleansEqual: AssertBooleansEqual,
	AssertCharactersEqual: AssertCharactersEqual,
	AssertStringEquals: AssertStringEquals,
	AssertNumberArraysEqual: AssertNumberArraysEqual,
	AssertBooleanArraysEqual: AssertBooleanArraysEqual,
	AssertStringArraysEqual: AssertStringArraysEqual,
	Negate: Negate,
	Positive: Positive,
	Factorial: Factorial,
	Round: Round,
	BankersRound: BankersRound,
	Ceil: Ceil,
	Floor: Floor,
	Truncate: Truncate,
	Absolute: Absolute,
	Logarithm: Logarithm,
	NaturalLogarithm: NaturalLogarithm,
	Sin: Sin,
	Cos: Cos,
	Tan: Tan,
	Asin: Asin,
	Acos: Acos,
	Atan: Atan,
	Atan2: Atan2,
	Squareroot: Squareroot,
	Exp: Exp,
	DivisibleBy: DivisibleBy,
	Combinations: Combinations,
	Permutations: Permutations,
	EpsilonCompare: EpsilonCompare,
	GreatestCommonDivisor: GreatestCommonDivisor,
	GCDWithSubtraction: GCDWithSubtraction,
	IsInteger: IsInteger,
	GreatestCommonDivisorWithCheck: GreatestCommonDivisorWithCheck,
	LeastCommonMultiple: LeastCommonMultiple,
	Sign: Sign,
	Max: Max,
	Min: Min,
	Power: Power,
	Gamma: Gamma,
	LogGamma: LogGamma,
	LanczosApproximation: LanczosApproximation,
	Beta: Beta,
	Sinh: Sinh,
	Cosh: Cosh,
	Tanh: Tanh,
	Cot: Cot,
	Sec: Sec,
	Csc: Csc,
	Coth: Coth,
	Sech: Sech,
	Csch: Csch,
	Error: Error,
	ErrorInverse: ErrorInverse,
	FallingFactorial: FallingFactorial,
	RisingFactorial: RisingFactorial,
	Hypergeometric: Hypergeometric,
	HypergeometricDirect: HypergeometricDirect,
	BernouilliNumber: BernouilliNumber,
	AkiyamaTanigawaAlgorithm: AkiyamaTanigawaAlgorithm,
	AddNumber: AddNumber,
	AddNumberRef: AddNumberRef,
	RemoveNumber: RemoveNumber,
	GetNumberRef: GetNumberRef,
	RemoveNumberRef: RemoveNumberRef,
	AddString: AddString,
	AddStringRef: AddStringRef,
	RemoveString: RemoveString,
	GetStringRef: GetStringRef,
	RemoveStringRef: RemoveStringRef,
	CreateDynamicArrayCharacters: CreateDynamicArrayCharacters,
	CreateDynamicArrayCharactersWithInitialCapacity: CreateDynamicArrayCharactersWithInitialCapacity,
	DynamicArrayAddCharacter: DynamicArrayAddCharacter,
	DynamicArrayCharactersIncreaseSize: DynamicArrayCharactersIncreaseSize,
	DynamicArrayCharactersDecreaseSizeNecessary: DynamicArrayCharactersDecreaseSizeNecessary,
	DynamicArrayCharactersDecreaseSize: DynamicArrayCharactersDecreaseSize,
	DynamicArrayCharactersIndex: DynamicArrayCharactersIndex,
	DynamicArrayCharactersLength: DynamicArrayCharactersLength,
	DynamicArrayInsertCharacter: DynamicArrayInsertCharacter,
	DynamicArrayCharacterSet: DynamicArrayCharacterSet,
	DynamicArrayRemoveCharacter: DynamicArrayRemoveCharacter,
	FreeDynamicArrayCharacters: FreeDynamicArrayCharacters,
	DynamicArrayCharactersToArray: DynamicArrayCharactersToArray,
	ArrayToDynamicArrayCharactersWithOptimalSize: ArrayToDynamicArrayCharactersWithOptimalSize,
	ArrayToDynamicArrayCharacters: ArrayToDynamicArrayCharacters,
	DynamicArrayCharactersEqual: DynamicArrayCharactersEqual,
	DynamicArrayCharactersToLinkedList: DynamicArrayCharactersToLinkedList,
	LinkedListToDynamicArrayCharacters: LinkedListToDynamicArrayCharacters,
	AddBoolean: AddBoolean,
	AddBooleanRef: AddBooleanRef,
	RemoveBoolean: RemoveBoolean,
	GetBooleanRef: GetBooleanRef,
	RemoveDecimalRef: RemoveDecimalRef,
	CreateLinkedListString: CreateLinkedListString,
	LinkedListAddString: LinkedListAddString,
	LinkedListStringsToArray: LinkedListStringsToArray,
	LinkedListStringsLength: LinkedListStringsLength,
	FreeLinkedListString: FreeLinkedListString,
	CreateLinkedListNumbers: CreateLinkedListNumbers,
	CreateLinkedListNumbersArray: CreateLinkedListNumbersArray,
	LinkedListAddNumber: LinkedListAddNumber,
	LinkedListNumbersLength: LinkedListNumbersLength,
	LinkedListNumbersIndex: LinkedListNumbersIndex,
	LinkedListInsertNumber: LinkedListInsertNumber,
	LinkedListSet: LinkedListSet,
	LinkedListRemoveNumber: LinkedListRemoveNumber,
	FreeLinkedListNumbers: FreeLinkedListNumbers,
	FreeLinkedListNumbersArray: FreeLinkedListNumbersArray,
	LinkedListNumbersToArray: LinkedListNumbersToArray,
	ArrayToLinkedListNumbers: ArrayToLinkedListNumbers,
	LinkedListNumbersEqual: LinkedListNumbersEqual,
	CreateLinkedListCharacter: CreateLinkedListCharacter,
	LinkedListAddCharacter: LinkedListAddCharacter,
	LinkedListCharactersToArray: LinkedListCharactersToArray,
	LinkedListCharactersLength: LinkedListCharactersLength,
	FreeLinkedListCharacter: FreeLinkedListCharacter,
	LinkedListCharactersAddString: LinkedListCharactersAddString,
	CreateDynamicArrayNumbers: CreateDynamicArrayNumbers,
	CreateDynamicArrayNumbersWithInitialCapacity: CreateDynamicArrayNumbersWithInitialCapacity,
	DynamicArrayAddNumber: DynamicArrayAddNumber,
	DynamicArrayNumbersIncreaseSize: DynamicArrayNumbersIncreaseSize,
	DynamicArrayNumbersDecreaseSizeNecessary: DynamicArrayNumbersDecreaseSizeNecessary,
	DynamicArrayNumbersDecreaseSize: DynamicArrayNumbersDecreaseSize,
	DynamicArrayNumbersIndex: DynamicArrayNumbersIndex,
	DynamicArrayNumbersLength: DynamicArrayNumbersLength,
	DynamicArrayInsertNumber: DynamicArrayInsertNumber,
	DynamicArrayNumberSet: DynamicArrayNumberSet,
	DynamicArrayRemoveNumber: DynamicArrayRemoveNumber,
	FreeDynamicArrayNumbers: FreeDynamicArrayNumbers,
	DynamicArrayNumbersToArray: DynamicArrayNumbersToArray,
	ArrayToDynamicArrayNumbersWithOptimalSize: ArrayToDynamicArrayNumbersWithOptimalSize,
	ArrayToDynamicArrayNumbers: ArrayToDynamicArrayNumbers,
	DynamicArrayNumbersEqual: DynamicArrayNumbersEqual,
	DynamicArrayNumbersToLinkedList: DynamicArrayNumbersToLinkedList,
	LinkedListToDynamicArrayNumbers: LinkedListToDynamicArrayNumbers,
	DynamicArrayNumbersIndexOf: DynamicArrayNumbersIndexOf,
	DynamicArrayNumbersIsInArray: DynamicArrayNumbersIsInArray,
	AddCharacter: AddCharacter,
	AddCharacterRef: AddCharacterRef,
	RemoveCharacter: RemoveCharacter,
	GetCharacterRef: GetCharacterRef,
	RemoveCharacterRef: RemoveCharacterRef,
	charToLowerCase: charToLowerCase,
	charToUpperCase: charToUpperCase,
	charIsUpperCase: charIsUpperCase,
	charIsLowerCase: charIsLowerCase,
	charIsLetter: charIsLetter,
	charIsNumber: charIsNumber,
	charIsWhiteSpace: charIsWhiteSpace,
	charIsSymbol: charIsSymbol,
	charCharacterIsBefore: charCharacterIsBefore,
	charDecimalDigitToCharacter: charDecimalDigitToCharacter,
	charCharacterToDecimalDigit: charCharacterToDecimalDigit,
	arraysStringToNumberArray: arraysStringToNumberArray,
	arraysNumberArrayToString: arraysNumberArrayToString,
	arraysNumberArraysEqual: arraysNumberArraysEqual,
	arraysBooleanArraysEqual: arraysBooleanArraysEqual,
	arraysStringsEqual: arraysStringsEqual,
	arraysFillNumberArray: arraysFillNumberArray,
	arraysFillString: arraysFillString,
	arraysFillBooleanArray: arraysFillBooleanArray,
	arraysFillNumberArrayRange: arraysFillNumberArrayRange,
	arraysFillBooleanArrayRange: arraysFillBooleanArrayRange,
	arraysFillStringRange: arraysFillStringRange,
	arraysCopyNumberArray: arraysCopyNumberArray,
	arraysCopyBooleanArray: arraysCopyBooleanArray,
	arraysCopyString: arraysCopyString,
	arraysCopyNumberArrayRange: arraysCopyNumberArrayRange,
	arraysCopyBooleanArrayRange: arraysCopyBooleanArrayRange,
	arraysCopyStringRange: arraysCopyStringRange,
	arraysIsLastElement: arraysIsLastElement,
	arraysCreateNumberArray: arraysCreateNumberArray,
	arraysCreateBooleanArray: arraysCreateBooleanArray,
	arraysCreateString: arraysCreateString,
	arraysSwapElementsOfNumberArray: arraysSwapElementsOfNumberArray,
	arraysSwapElementsOfStringArray: arraysSwapElementsOfStringArray,
	arraysReverseNumberArray: arraysReverseNumberArray,
	test: test
};